#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Util::Number;
use Sequence::Domain;
use Util::Array;


my $verbose    	= "false";
my $domainsFile = "";
my $familyACC	= "";
my $cutOffEv 	= "1";
my $cutOffExpEv = "5";
my $tmpDir = "";
my $logBase = 10;
my $DOMAIN_COV_THRESHOLD = 40;
my $svm = "false";
my $overlapAA_alowed = "30"; 
my $perc_overlap_alowed= "0.5";
my $archNumber = 1;
my $cutOFFWeakPred = "false";
my $overlapList = "";
my $DCOList = "";
my $outputFile = "";
my $accNOrganism =  "";
my $N =  1;
my $cutOffProb = 0.05;
my $evalueCutOffConf = 1e-05;
my $SVM_CUTOFF = 0.1;
my $review = "false";

my $COL_EVALUE 	= 0;
my $COL_Q_ACC	= 3;
my $COL_D_ACC	= 4;
my $COL_H_START	= 1;
my $COL_H_END	= 2;
my $COL_Q_SIZE  = 5;

my $domains_overlap = "";
my @array_domains = ();
my $smaller;
my %hash_DCO = ();
my %hash_domain = ();
my $cutOffsFile = "";

my %hashSVMcut = ();

my $PROG_NAME = "CASH_27::scripts::archs::findArchCASH.pl";


&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-domainsFile"){
			$domainsFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-domCov"){
			$DOMAIN_COV_THRESHOLD = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-overlapAA_alowed"){
			$overlapAA_alowed = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-perc_overlap_alowed"){
			$perc_overlap_alowed = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-svm"){
			$svm = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-cutOFFWeakPred"){
			$cutOFFWeakPred = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-overlapList"){
			$overlapList = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-DCOList"){
			$DCOList = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNOrganism"){
			$accNOrganism = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-N"){
			$N = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-SVM_CUTOFF"){
			$SVM_CUTOFF = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-evalueCutOffConf"){
			$evalueCutOffConf = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-review"){
			$review = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-cutOffsSVMFile"){
			$cutOffsFile = $ARGV[$counter+1];
		}							
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($domainsFile, "ERROR::".$PROG_NAME.":: Parameter -domainsFile is required\n");
	FilesIO->checkFile($domainsFile, "ERROR::".$PROG_NAME.":: File NOT found ".$domainsFile);	
	String->isEmpty($DCOList, "ERROR::".$PROG_NAME.":: Parameter -DCOList is required\n");
	FilesIO->checkFile($DCOList, "ERROR::".$PROG_NAME.":: File NOT found ".$DCOList);	
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME."::  Parameter -verbose must be true or false!");
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having temporary directory is required!");
	String->isEmpty($overlapList, "ERROR::".$PROG_NAME.":: Parameter -overlapList having domains with allowed overlaps");
	FilesIO->checkFile($overlapList, "ERROR::".$PROG_NAME.":: File NOT found ".$overlapList);
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required");
		
	if ($svm eq "true"){
		$archNumber = 2;
		$cutOffExpEv = 0.05;
		String->isEmpty($cutOffsFile, "ERROR::".$PROG_NAME.":: Parameter -cutOffsSVMFile is required ");
		FilesIO->checkFile($cutOffsFile, "ERROR::".$PROG_NAME.":: File NOT found ".$cutOffsFile);
	}
	
	print "-domainsFile = $domainsFile\n";
	print "-DCOList = $DCOList\n";	
	print "-domCov = $DOMAIN_COV_THRESHOLD\n";
	print "-verbose = $verbose\n";
	print "-overlapAA_alowed = $overlapAA_alowed\n";
	print "-perc_overlap_alowed = $perc_overlap_alowed\n";
	print "-svm = $svm\n";
	print "-cutOFFWeakPred = $cutOFFWeakPred\n";
	print "-overlapList = $overlapList\n";
	print "-outputFile = $outputFile\n";
	print "-tmpDir = $tmpDir\n";
	print "-accNOrganism = $accNOrganism\n";
	print "-cutOffsFile = $cutOffsFile\n";
	
	print "ok\n";	
}

############################ sub main ##################################
sub main{
	my $arch = "";
	$domains_overlap = FilesIO->lineFileToString($overlapList);
	%hash_DCO = FilesIO->fileToHashConcatKey($DCOList.".pairs", "\t", "0,1");
	
	if ($svm eq "true"){
		%hashSVMcut = FilesIO->fileToHash($cutOffsFile, "\t", 0);
	}
	
	#first iteration
	$arch = findBestArch(0, $arch);
	
	#Second iteration
	#$arch = findBestArch(0.1, $arch);
	#$arch = transladeEv($arch);
	FilesIO->saveFile($outputFile, $arch, $verbose);
}
############################ sub findBestArch ###########################
sub findBestArch{
	my $factorCutoff = $_[0];
	my $bestArch 	 = $_[1];
	
	my $arch = "";				my @array_arch = ();
	my $best;	
	
	@array_domains = FilesIO->fileToArray2d($domainsFile, "\t");
	%hash_domain = FilesIO->fileToHashConcatKey($domainsFile, "\t", "2,3,5");
	if ($verbose eq "true"){print  "Total domains : ". scalar(@array_domains)."\n";}
	&removeSmallDomain;
	if ($verbose eq "true"){print  "Total after removing small domains : ". scalar(@array_domains)."\n";}
	
	if ($svm eq "true" && scalar(@array_domains) > 0){
		&removeDomainSVMscore($factorCutoff);
		if ($verbose eq "true"){print  "Total after removing by SVM score : ". scalar(@array_domains)."\n";}
	}

	if (scalar(@array_domains) > 0){
		if ($verbose eq "true"){print  Array->printMatrix2("\t", "\n", \@array_domains);}
		formatDomains();
		
		@array_arch = &analiseArch($bestArch);
		
		if (scalar(@array_arch) > 0) {
			@array_arch = Array->sortArray2d(0, \@array_arch);
			if ($verbose eq "true"){printAllArchs(\@array_arch);}
			
			$best = &getBestArch(\@array_arch);					
			$arch = $array_arch[$best][4];
			if ($review eq "true"){
				$arch = &doRevision($arch);
			}
		}
	}else{
		print "ERROR::".$PROG_NAME.":: $accNOrganism :: $domainsFile has 0 domains greater than $DOMAIN_COV_THRESHOLD % \n";
		exit;
	}	
	return $arch;
}
############################ sub main ##################################
sub printAllArchs{
	my ($archs) = @_;
	my @larchs = @{$archs};
	print "\n==>All archrs\n";
	for (my $i = scalar(@larchs) -1; $i >=0; $i--){
		print "$i Score\t".$larchs[$i][0]."\tDCO=".$larchs[$i][1]."\tPairDCO=".$larchs[$i][2]."\tAVG(score)=".$larchs[$i][3]."\n";
		print $larchs[$i][4]."\n";
	}	
}
############################ sub main ##################################
sub removeSmallDomain{
	
	my @array_domains_clean = ();
	my $domain_cov;	
	my $evalue;
	my $prob;
	
	for (my $j = 0; $j < scalar(@array_domains); $j++){
		$domain_cov = Domain->calculateCoverage($array_domains[$j][7], $array_domains[$j][8], $array_domains[$j][2], $array_domains[$j][3]);
		#print  $array_domains[$j][0]."\t". $array_domains[$j][7]."\t".$array_domains[$j][8]."\t". $array_domains[$j][2]."\t".$array_domains[$j][3]."\n";
		#print "Domain Cov ".$domain_cov."\n";
		$evalue = $array_domains[$j][0];
		if ($domain_cov >= $DOMAIN_COV_THRESHOLD){	
			push (@array_domains_clean, $array_domains[$j]);
		}elsif ($evalue <= $evalueCutOffConf){
			push (@array_domains_clean, $array_domains[$j]);
		}		
	}
	@array_domains = @array_domains_clean;
}

############################ sub main ##################################
sub removeDomainSVMscore{
	
	my $factorCutOff	= $_[0];
	
	my @array_domains_clean = ();
	my $domain_cov;	
	my $domain;
	my $evalue;
	my $svm_score;
	
	for (my $j = 0; $j < scalar(@array_domains); $j++){
		$evalue = $array_domains[$j][0];
		$svm_score = $array_domains[$j][1];
		$svm_score = sprintf("%.2f", $svm_score);
		$domain =  $array_domains[$j][5];
		#print  $array_domains[$j][0]."\t". $array_domains[$j][7]."\t".$array_domains[$j][8]."\t". $array_domains[$j][2]."\t".$array_domains[$j][3]."\n";
		#print "Domain Cov ".$domain_cov."\n";
		
		if (exists($hashSVMcut{$domain})){
			$SVM_CUTOFF = String->getElementArrayLine($hashSVMcut{$domain}, "\t", 1);
			$SVM_CUTOFF = sprintf("%.2f", $SVM_CUTOFF);
			if ($factorCutOff > 0){
				$SVM_CUTOFF = $SVM_CUTOFF - $factorCutOff;
				if ($SVM_CUTOFF < 0.1){
					$SVM_CUTOFF = 0.1;
				}
			}			
			#print "svm_score = ".$svm_score."\tSVM_CUTOFF=".$SVM_CUTOFF."\n";
			if ($svm_score > $SVM_CUTOFF){
				push (@array_domains_clean, $array_domains[$j]);
			}
		}else{				
			print $PROG_NAME."::ERROR $domain NOT found in $cutOffsFile\n";
			exit;
		}
		
	}
	@array_domains = @array_domains_clean;
}


############################ sub main ##################################
sub formatDomains{	
	@array_domains = Array->getSomeColsMatrix2("0:2:3:4:5:6:7:8:1:9", \@array_domains);
	@array_domains = Array->logMatrizByCol($logBase, 0, \@array_domains);	
	@array_domains = Array->multiplyValueMatrizByCol(-1, 0, \@array_domains);	
	@array_domains = transladeScore(0, \@array_domains);	 
	@array_domains = Array->sortArray2d(1, \@array_domains);
}

############################ sub main ##################################
sub transladeScore{	
	my ($col, $matrix) = @_;
	my @lmatrix = @{$matrix};
	
	my @array_aux = ();
	@array_aux = Array->copyColMatrixToArray(scalar(@lmatrix), $#{$lmatrix[$col]} + 1, $col, \@lmatrix);
	
	$smaller = Array->getSmaller(\@array_aux);	
	if ($smaller < 0){
		$smaller = $smaller*(-1);
		@lmatrix = Array->sumValueMatrizByCol($smaller, 0, \@lmatrix);
		$cutOffExpEv = $cutOffExpEv + $smaller;
	}
	
	return @lmatrix;
}

############################ sub main ##################################
sub analiseArch{
	my $bestArch 	 = $_[0];
	
	my $domain;					my $start;
	my $end;					my $key;
	my $evalue;					my $accCand;
	my $accProt;				my $startProt;
	my $endProt;				my @array_aux = ();
	my $ev_old;					my $label = "N_";
	my $count = 0;				my @arrayNoOverlap_i;
	my @arrayNoOverlap;			my $info_arch = "";
	my $score;					my $domainName;
	my $rest;					my $prob;
	my @bestArchDomains = ();	my $acceptable;
	my @array_arch = ();
	
	my $total = scalar(@array_domains);
	
	if ($bestArch ne ""){
		@bestArchDomains = String->StringToArray2d($bestArch, "\n", "\t");
	}
	
	for (my $i = 0; $i < scalar(@array_domains); $i++){
		$evalue 	= $array_domains[$i][0];
		$start  	= $array_domains[$i][1];
		$end    	= $array_domains[$i][2];
		$accCand	= $array_domains[$i][3];
		$domain 	= $array_domains[$i][4];
		$accProt 	= $array_domains[$i][5];
		$startProt  = $array_domains[$i][6];
		$endProt    = $array_domains[$i][7];
		$domainName = $array_domains[$i][8];
		$prob 		= $array_domains[$i][9];
		
		$acceptable = "true";			
	
		if (scalar(@bestArchDomains) > 0 && Array->isInColMatrix(4, $domain, \@bestArchDomains) ==-1 ){
			$acceptable = "false";
		}	
			
		if ($acceptable eq "true"){
			@arrayNoOverlap_i = Domain->removePartialOverlapHit($start, $end, $domain, $overlapAA_alowed, $perc_overlap_alowed, $domains_overlap, \@array_domains);	
			#print "domain $i ". scalar(@array_domains)."==> $domain $prob \n";	
			if (!defined($domainName)){
				$domainName = "";
			}
	
			@arrayNoOverlap = removeOverlapHitMaxDCO($domain, "true", \@arrayNoOverlap_i);
			$info_arch = $evalue."\t".$start."\t".$end."\t".$accCand ."\t".$domain."\t".$accProt."\t".$startProt."\t".$endProt."\t".$domainName."\t".$prob."\n";		
			$info_arch .=  Array->printMatrix2("\t", "\n", \@arrayNoOverlap);		
			if ($verbose eq "true"){print "\n----ARch----\n$info_arch";}	
		
			if ($info_arch ne "" && existsArch($info_arch, \@array_arch) eq "false"){
				#print "calculateScore\n";
				@array_aux = calculateScore($info_arch);
				if ($verbose eq "true"){print ">>>>>".$array_aux[4]};
				#print "SC0RE= ".$array_aux[0]."\n";
			
				if (scalar(@array_aux) > 0){
					$array_arch[$count][4] = $array_aux[4];
					$array_arch[$count][3] = $array_aux[3];
					$array_arch[$count][2] = $array_aux[2];
					$array_arch[$count][1] = $array_aux[1];
					$array_arch[$count][0] = $array_aux[0];
					$count++;
				}
			}else{
				if($verbose eq "true"){print "Arch already included\n";}
			}				
		}
	}	
	return @array_arch;
	
}



############################ sub runFormatFiletoSam ##################################
sub removeOverlapHitMaxDCO{
	my ($domain_t, $onlyPairs, $array) = @_;
	my @localArray = @{$array};
	@localArray = Array->sortArray2dDesc(0, \@localArray);
	
	my $startHit_i;				my $startHit_j;
	my $endHit_i;				my $endHit_j;
	my $evalue_i;				my $evalue_j;
	my $save = "true";			my $domain_i;
	my $info = "";				my $domain_j;
	my @array_ret = ();			my $stop;
	my $i =0;					my  $j;
	my $DCO_i;					my $DCO_j;
	my $domain_name_i;			my $domain_name_j;
	my $svm_i;					my $svm_j;
	
	while ( $i < scalar(@localArray)) {
		$startHit_i = $localArray[$i][1];
		$endHit_i = $localArray[$i][2];
		$evalue_i = $localArray[$i][0];
		$domain_i = $localArray[$i][4];
		$svm_i	= $localArray[$i][8];
		if ($svm eq "true"){
			$evalue_i = $svm_i;
		}		
		$domain_name_i = $localArray[$i][9];
		#print "\n\nNEW i=$i == > $domain_i $domain_name_i $startHit_i $endHit_i $evalue_i\n";
		$DCO_i = countDCO($domain_t." ".$domain_i);	
		#print "DCO_i = $DCO_i\n";
		$save = "true";	
		$j =0;
		
		$stop = "false";
		if ($DCO_i > 0){
			while ($j < scalar(@localArray) && $stop eq "false") {
				if ($i != $j){
					$startHit_j = $localArray[$j][1];
					$endHit_j = $localArray[$j][2];	
					$evalue_j = $localArray[$j][0];		
					$domain_j = $localArray[$j][4];	
					$svm_j		= $localArray[$j][8];
					if ($svm eq "true"){
						$evalue_j = $svm_j;
					}
					$domain_name_j = $localArray[$j][9];
						
					#print "j=$j $domain_j $domain_name_j $startHit_j $endHit_j $evalue_j\n";			
					#print "OVERLAP".Blast->isOverlapHit($startHit_i, $endHit_i, $startHit_j, $endHit_j, $overlapAA_alowed, $perc_overlap_alowed, $domains_overlap, $domain_i, $domain_j)."\n";
					if (Domain->isOverlapHit($startHit_i, $endHit_i, $startHit_j, $endHit_j, $overlapAA_alowed, $perc_overlap_alowed, $domains_overlap, $domain_i, $domain_j) eq "true" ){	
						$DCO_j = countDCO($domain_t." ".$domain_j);	
					#	print "DCO_j=$DCO_j\n";		
					#	print "evalue_i = $evalue_i  evalue_j = $evalue_j\n";	
					#	print "svm_i = $svm_i  svm_j = $svm_j\n";	
						if ($DCO_j > 0){
							if ($evalue_i < $evalue_j){
								$save = "false";					
								$stop = "true";		
					#			print " found better\n";
							}
						} 
					}
				}
				$j++;
			}
			if ($save eq "true"){
				#print "save $i\n";
				push(@array_ret, $localArray[$i]);
			}
		}
		$i++;
	}
	return @array_ret;
}

############################ sub main ##################################
sub countDCO{
	my $domains 	= $_[0];
	
	my @array = String->StringToArray($domains, " +");
	
	if (scalar(@array) == 2){		
		return hasPairDCO($array[0], $array[1]);
	}	
	#return hasMultDCO($domains);	
	return Domain->hasMDO($domains, $DCOList, $tmpDir, $verbose);	
}

############################ sub main ##################################
sub hasPairDCO{
	my $domain_j 	= $_[0];
	my $domain_k 	= $_[1];
	
	$domain_j =~ s/PF//g;
	$domain_k =~ s/PF//g;
	
	if (exists($hash_DCO{$domain_j.$domain_k}) ||  exists($hash_DCO{$domain_k.$domain_j})){
		return 1;
	}
	return 0;	
}
############################ sub main ##################################
sub hasMultDCO{
	my $domains		= $_[0];
		
	$domains = "\"$domains\"";
	$domains =~ s/PF//g;
	
	my $tmpFile = $tmpDir."tmp.txt";
	
	my $cmd = $ENV{'PERL_LIB'}."/run/socket/./findClient $domains > $tmpFile";
	#print $cmd."\n";
	system($cmd);
	FilesIO->checkFile($tmpFile, "ERROR::".$PROG_NAME.":: File NOT created".$tmpFile);
	
	my $totalDCO = FilesIO->lineFileToString($tmpFile);
	$totalDCO = String->removeCaracter($totalDCO, "\n");
	
	return  $totalDCO;	
}

############################ sub main ##################################
sub hasMultDCO2{
	my $domains		= $_[0];
		
	$domains = "\"$domains\"";
	$domains =~ s/PF//g;
	
	my $tmpFile = $tmpDir."tmp.txt";
	
	my $cmd = $ENV{'PERL_LIB'}."/run/socket/./findClient $domains > $tmpFile";
	#print $cmd."\n";
	system($cmd);
	FilesIO->checkFile($tmpFile, "ERROR::".$PROG_NAME.":: File NOT created".$tmpFile);
	
	my $totalDCO = FilesIO->lineFileToString($tmpFile);
	$totalDCO = String->removeCaracter($totalDCO, "\n");
	
	return  $totalDCO;	
}

############################ sub main ##################################
sub existsArch{
	my ($info_arch_new, $archs) = @_;
	
	my @array_arch = @{$archs};
	
	my $info_arch1;				my $info_arch2;
	my $score_i;				my $score_new;
	
	my @archArrayLine = ();
	my @archNewArrayLine = String->StringToArray2d($info_arch_new, "\n", "\t");
	@archNewArrayLine = Array->getSomeColsMatrix2("1:2:4", \@archNewArrayLine);
	$info_arch1 = Array->printMatrix2("\t", "\n", \@archNewArrayLine);
	#print "info_arch1".$info_arch1."\n"; 
	
	if (scalar(@array_arch) == 0){
		return "false";
	}else{
		#print "scalar(array_arch)".scalar(@array_arch)."\n";
		for (my $i = 0; $i < scalar(@array_arch); $i++){
			@archArrayLine = String->StringToArray2d($array_arch[$i][4], "\n", "\t");
			@archArrayLine = Array->getSomeColsMatrix2("1:2:4", \@archArrayLine);
			$info_arch2 = Array->printMatrix2("\t", "\n", \@archArrayLine);		
			#print "info_arch2".$info_arch2."\n"; 	
			if (compareArchs($info_arch2, $info_arch1) eq "true"){	
				return "true";
			}			
		}	
	}
	return "false";
}

############################ sub main ##################################
sub compareArchs{
	my $arch1 	= $_[0];
	my $arch2 	= $_[1];
	
	my $count_same = 0;
	
	
	
	my @archArray1 = String->StringToArray($arch1, "\n");
	my @archArray2 = String->StringToArray($arch2, "\n");
	
	$arch1 =~ s/\n/ /g;
	$arch2 =~ s/\n/ /g;
	#print ">>compare\n$arch1\n\n$arch2\n\n";
	
	for (my $j = 0; $j < scalar(@archArray1); $j++){					
		for (my $k = 0; $k < scalar(@archArray2); $k++){
			if ($archArray2[$k] eq $archArray1[$j]){
				$count_same++;
				$k = scalar(@archArray2);
			}
		}	
	}
	if ($count_same == (scalar(@archArray2))){
		#print "Archs are equals\n";
		return "true";
	}
	#print "Archs NOT are equals\n";
	return "false";
}

############################ sub runFormatFiletoSam ##################################
sub calculateScore{
	my ($arch) = @_;
	my $info;				my $comb = 2;
	my @array_comb;			my @array_comb_new = ();
	my $i;					my @array_aux = ();
	my %hashA = ();			my %hashB = ();	
	my $count;				my $DCO;
	my $stop = "false";		my $new_arch = "";
	my $maxScore = -1;		my $domain_j;
	my $score;				my @arrayRet = ();
	my $maxDCO;				my $domain_i;
	my $DCO_i;				my $svm_scores;
	my $POS_SCORE = 0;
	
	if ($svm eq "true"){
		$POS_SCORE = 8;
	}
	
	my @archArray = String->StringToArray2d($arch, "\n", "\t");
	my $sizeArch = scalar(@archArray);
	my @archArray_aux =  Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 4, \@archArray);
	#@archArray_aux = sort(@archArray_aux);
	#print "Arch array =" . Array->printArray("\t", \@archArray_aux)."\n";
	#exit;
	@archArray_aux = Array->removeRepeated(\@archArray_aux);
	@archArray_aux = sort(@archArray_aux);
	if (scalar(@archArray_aux) == 1){
		$info =  Array->printArray(" ", \@archArray_aux);
	}
	else {
		@archArray_aux =  Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 4, \@archArray);
		$info =  Array->printArray(" ", \@archArray_aux);
	}	
	
	@array_comb = Domain->domainCombination($tmpDir, $info, $comb, $verbose);

	for ($i = 0; $i < scalar(@array_comb); $i++){
		$DCO = countDCO($array_comb[$i]);
		if ($DCO > 0){
			$hashA{$array_comb[$i]} = $DCO;
		}	
	}
	
	if (keys(%hashA) == 0){
		#print "archArray".scalar(@archArray)."\n";
		if (scalar(@archArray) == 1){
			$stop = "true";
			$hashA{$archArray_aux[0]} = 0;	
		}else{
			$DCO = countDCO($archArray_aux[0]." ".$archArray_aux[0]);
			$hashA{$archArray_aux[0]." ".$archArray_aux[0]} = $DCO;	
		}				
	}
		
	while ($stop eq "false"){
		#print "Start $i\n";
		while ( my ($key, $value) = each(%hashA) ) {
			#print "HASHA $key\n";
			@array_aux = split(/ +/, $key);
			$comb = scalar(@array_aux)-1;
			@array_comb = Domain->domainCombination($tmpDir, $key, $comb, $verbose);
			#print "\narray_comb\n".Array->printArray("\n", \@array_comb)."\n";
		
			for (my $j = 0; $j < scalar(@archArray_aux); $j++){
				#print "archArray_aux = ". $archArray_aux[$j]."\n";
				@array_aux = (); $count=0;
				for (my $k = 0; $k < scalar(@array_comb); $k++){
					$info = String->sort($array_comb[$k]." ".$archArray_aux[$j], " +", " ");
					#print "info = $info\n";
					if (exists($hashA{$info})){
						$count++;
						#print "count = $count\n";
					}
				}
				#print "----->count = $count\t scalar(array_comb)".scalar(@array_comb)."\n";
				if ($count == scalar(@array_comb)){					
					$info = $key." ".$archArray_aux[$j];	
					@array_aux = split(/ +/, $info);				
					$info = String->sort($info, " +", " ");
					if (!exists($hashB{$info})){
						$DCO = countDCO($info);
						if ($DCO > 0 && scalar(@array_aux) <= $sizeArch){
							$hashB{$info} = $DCO;
							#print "hashB = $info = $DCO\n";
						}
					}
				}
			}			
		}
		if (keys(%hashB) == 0){
			$stop = "true";
		}else{
			%hashA =%hashB;
			%hashB = ();
		}
	}	
	
	my $sum_scores = 0; $maxDCO = 0; my $pairDCO= 0;
	
	while ( my ($key, $value) = each(%hashA)) {
		@array_aux = split(/ +/, $key);
		$maxDCO = scalar(@array_aux);
	}
	
	
	@array_aux = Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, $POS_SCORE, \@archArray);
	$maxScore = Array->getBigger(\@array_aux);
	@array_aux = Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, $POS_SCORE, \@archArray);
	$sum_scores = Array->getAverage(\@array_aux);
	@array_aux = Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 8, \@archArray);
	$svm_scores = Array->getAverage(\@array_aux);
	#$sum_scores = $sum_scores + $svm_scores;
	
	for (my $i = 0; $i < scalar(@archArray); $i++){
		$domain_i = $archArray[$i][4];
		for (my $j = $i+1; $j < scalar(@archArray); $j++){
			$domain_j = $archArray[$j][4];
			$DCO_i = countDCO($domain_i." ".$domain_j);		
			if ($DCO_i > 0){
				$pairDCO++;
			}					
		}
	}
	
	if (scalar(@archArray) > 0){
		$arrayRet[0] = $maxScore;
		$arrayRet[1] = $maxDCO;
		$arrayRet[2] = $pairDCO;
		$arrayRet[3] = $sum_scores;
		$arrayRet[4] = $arch;
		#print "MAXSCORE $maxScore; MAXDCO =$maxDCO; MAXPAIRS=$pairDCO AVG=$sum_scores \n ";
	}

	return @arrayRet;	
}


############################ sub runFormatFiletoSam ##################################
sub calculateScore2{
	my ($arch) = @_;
	my $info;				my $comb = 2;
	my @array_comb;			my @array_comb_new = ();
	my $i;					my @array_aux = ();
	my %hashA = ();			my %hashB = ();	
	my $count;				my $DCO;
	my $stop = "false";		my $new_arch = "";
	my $maxScore = -1;		my $domain_j;
	my $score;				my @arrayRet = ();
	my $maxDCO;				my $domain_i;
	my $DCO_i;		
	my $sum_scores = 0; $maxDCO = 0; my $pairDCO= 0;
	
	my @archArray = String->StringToArray2d($arch, "\n", "\t");
	my $sizeArch = scalar(@archArray);
	my @archArray_aux =  Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 4, \@archArray);
	#@archArray_aux = sort(@archArray_aux);
	@archArray_aux = sort(@archArray_aux);
	@archArray_aux = Array->removeRepeated(\@archArray_aux);
	
	print "OI Arch array =" . Array->printArray("\t", \@archArray_aux)."\n";
	$info = Array->printArray(" ", \@archArray_aux);
	$maxDCO = Domain->hasMDO($info, $DCOList, $tmpDir, $verbose);
		
	@array_aux = Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 0, \@archArray);
	$maxScore = Array->getBigger(\@array_aux);
	@array_aux = Array->copyColMatrixToArray(scalar(@archArray), $#{$archArray[0]} + 1, 0, \@archArray);
	$sum_scores = Array->getAverage(\@array_aux);
	
	for (my $i = 0; $i < scalar(@archArray); $i++){
		$domain_i = $archArray[$i][4];
		for (my $j = $i+1; $j < scalar(@archArray); $j++){
			$domain_j = $archArray[$j][4];
			$DCO_i = countDCO($domain_i." ".$domain_j);		
			if ($DCO_i > 0){
				$pairDCO++;
			}					
		}
	}
	
	if (scalar(@archArray) > 0){
		$arrayRet[0] = $maxScore;
		$arrayRet[1] = $maxDCO;
		$arrayRet[2] = $pairDCO;
		$arrayRet[3] = $sum_scores;
		$arrayRet[4] = $arch;
	}
	#print $arch."\n".$maxScore."\n".$maxDCO."\n".$pairDCO."\n".$sum_scores."\n";
	
	return @arrayRet;	
}


############################ sub main ##################################
sub getBestArch{
	my ($archs) = @_;
	my @array_arch = @{$archs};
	my $range;
	my $best = scalar(@array_arch) -1;
	my $cutPhi;
	
	if ($svm eq "true"){
		$cutPhi = 0.1;
	}else{
		$cutPhi = 10;
	}
		
	for (my $i = scalar(@array_arch) -2; $i >=0; $i--){
		$range = $array_arch[$best][0] - $array_arch[$i][0];
		
		if($verbose eq "true"){print "Archs $i $best $range\n";}
		if ($range <= $cutPhi){
			#print "Archs $i ".$array_arch[$i][1]." $best ". $array_arch[$best][1] ."\n";
			if ($array_arch[$i][1] > $array_arch[$best][1]){
				$best = $i
			}elsif ($array_arch[$i][1] == $array_arch[$best][1]){
				if ($array_arch[$i][2] > $array_arch[$best][2]){
					$best = $i
				}elsif($array_arch[$i][2] == $array_arch[$best][2]){
					if ($array_arch[$i][3] > $array_arch[$best][3]){
						$best = $i
					}
				}
			}
		}
		if($verbose eq "true"){print "Archs win  $best\n";}
	}
	return $best;	
}
############################ sub main ##################################
sub transladeEv{
	my $info_arch 	= $_[0];
	my @archArrayLine = String->StringToArray2d($info_arch, "\n", "\t");
	my $evalue;				my $key;
	
	for (my $i = 0; $i < scalar(@archArrayLine); $i++){
		$key = $archArrayLine[$i][1].$archArrayLine[$i][2].$archArrayLine[$i][4];
		if (!exists($hash_domain{$key})){
			print $PROG_NAME."::ERROR:: $key NOT found in $domainsFile\n";
			exit;
		}
		$evalue = String->getElementArrayLine($hash_domain{$key}, "\t", 0);
		$archArrayLine[$i][0] = $evalue;
	}
	
	return Array->printMatrix2("\t", "\n", \@archArrayLine);
}



############################ sub main ##################################
sub doRevision{
	my $arch	= $_[0];
	
	my $domain;				my $start;				my $end;
	
	my @array_aux = String->StringToArray2d($arch, "\n", "\t");

	#remove domains previouly selected from arch
	print "\n\n". $arch."\n\n";
	
	print "\n\n". Array->printMatrix2("\t", "\n", \@array_domains);
	for (my $i = 0; $i < scalar(@array_aux); $i++){
		$start = $array_aux[$i][$COL_H_START];
		$end   =  $array_aux[$i][$COL_H_END];
		$domain=  $array_aux[$i][$COL_D_ACC];
		print ">> $domain $start $end\n";
		@array_domains = Domain->removePartialOverlapHit($start, $end, $domain, $overlapAA_alowed, $perc_overlap_alowed, $domains_overlap, \@array_domains);
	}
	print "Before Review\n\n". Array->printMatrix2("\t", "\n", \@array_domains);
	if (scalar(@array_domains) > 0){
		push(@array_aux, @array_domains);
		@array_aux = Domain->removeOverlapHitMax($overlapAA_alowed, $perc_overlap_alowed, $domains_overlap, \@array_aux);		
	}
	return Array->printMatrix2("\t", "\n", \@array_aux);
}



