#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Database::PFAM;
use Util::Array;
use Util::Hash;
use Align::Primary::blast;

my $outputFile = "";
my $verbose    	= "false";
my $taxonDir = "";
my $resultFile  = ""; 
my $resultFileBest = "";
my $totalModels = "";
my $tmpDir = "";
my $domainAvgFile = "";
my $PFAM_ACC = "";
my $level = 4;
my $filtreEv = 1;
my $domCov = 0.2;
my $TEV = "1e-20";
my $test = "false";
my $PERC_OVERLAP = "0.5";
my $mappingTaxon = "";

my $filtre = "false";
my $EV_CUT_OFF = 100;
my $seqListFile = "";
my $MAX_EVALUE = 100;
my $GENERAL_MODEL = "HMMer-3";
my $GENERAL_SPECIE = "ALL";
my $TSPECIE = "0.25";
my $specieDistanceFile = "";
my $resultSpecieFile = "";
my $MAX_DISTANCE = 7583;

my $taxonModelFile = "";
my $taxonTestFile  = "";

my $COL_GA = 3;
my $COL_BITS_RESULT = 7;
my $COL_MODEL = 0;
my $COL_EVALUE = 4;

my $taxonRepDir;


my $PROG_NAME = "CASH_27::scripts::ensemble::createAtributtesVector.pl";

my %hash_taxon = ();	

my %hash_overlap = ();	
my $count_models;	
my @array_cev = (); 
my @array_sp_model = ();	

my @array_models = ();
my %hash_species_distance = ();
my @specieDistance_array = ();
my @array_mediana = ();

my %hash_avg = ();
my %hash_best = ();
my %hash_result = ();

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-resultFileBest"){
			$resultFileBest = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-resultFile"){
			$resultFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-filtre"){
			$filtre = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-taxonDir"){
			$taxonDir = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-totalModels"){
			$totalModels = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-test"){
			$test = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-domCov"){
			$domCov = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-level"){
			$level = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-filtreEv"){
			$filtreEv = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-domainAvgFile"){
			$domainAvgFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-PFAM_ACC"){
			$PFAM_ACC = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-PERC_OVERLAP"){
			$PERC_OVERLAP = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-mappingTaxon"){
			$mappingTaxon = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-taxonModelFile"){
			$taxonModelFile = $ARGV[$counter+1];		
		}								
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	

	String->isEmpty($resultFile, "ERROR::".$PROG_NAME.":: Parameter -resultFile having Result file is required!");
	FilesIO->checkFile($resultFile, "ERROR::".$PROG_NAME.":: File NOT found".$resultFile);
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required!");
	String->isEmpty($resultFileBest, "ERROR::".$PROG_NAME.":: Parameter -resultFileBest having Best Result file is required!");
	FilesIO->checkFile($resultFileBest, "ERROR::".$PROG_NAME.":: File NOT found ".$resultFileBest);
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME."::Parameter -verbose must be true or false!");
	String->isEmpty($totalModels, "ERROR::".$PROG_NAME.":: Parameter -totalModels having total of models required!");
	String->isEmpty($taxonDir, "ERROR::".$PROG_NAME.":: Parameter -taxonDir having taxon directory is required!");
	FilesIO->checkDirectory($taxonDir, "ERROR::".$PROG_NAME.":: Directory NOT found".$taxonDir);
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having temporary directory is required!");
	String->isEmpty($domainAvgFile, "ERROR::".$PROG_NAME.":: Parameter -domainAvgFile having average domain size is required!");
	FilesIO->checkFile($domainAvgFile, "ERROR::".$PROG_NAME.":: File NOT found".$domainAvgFile);
	String->isEmpty($PFAM_ACC, "ERROR::".$PROG_NAME.":: Parameter -PFAM_ACC having PFAM accession number is required!");
	String->isEmpty($taxonModelFile, "ERROR::".$PROG_NAME.":: Parameter -taxonModelFile having Taxon Model file is required!");
	FilesIO->checkFile($taxonModelFile, "ERROR::".$PROG_NAME.":: File NOT found".$taxonModelFile);
	
	
	String->isEmpty($mappingTaxon, "ERROR::".$PROG_NAME.":: Parameter -mappingTaxon having Mapping Taxon file is required!");
	FilesIO->checkFile($mappingTaxon, "ERROR::".$PROG_NAME.":: File NOT found".$mappingTaxon);
	%hash_taxon = FilesIO->fileToHash($mappingTaxon, "\t", 0);
		
	print "-resultFile = $resultFile\n";
	print "-resultFileBest = $resultFileBest\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-totalModels = $totalModels\n";
	print "-taxonDir = $taxonDir\n";
	print "-tmpDir = $tmpDir\n";
	print "-domainAvgFile = $domainAvgFile\n";
	print "-mappingTaxon = $mappingTaxon\n";
	print "-PFAM_ACC = $PFAM_ACC\n";
	
	print "ok\n";	
}	

############################ sub checkParameters ##################################
sub main {
	my $starHit;					my $endHit;						
	my $info = "";					my $accNumber;
	my @array_aux = ();				my $evalue;
	my $all_results;				my @array_overlap;
	my $new_key;					my $perc_model;
	my $info_att =  "";				my $head = "";
	my $specieExample;				my $Total;
 	my $mediana;					my $distance_file_index;
 	my $distance_file;				my $size;
 	my $avg_domain;					my @array_models;
 	my $accModel;					my $perc_clade;
 	my $accNumber_clean;			my $count = 0;
 
 
	%hash_best = FilesIO->fileToHashConcatKey($resultFileBest, "\t", "3,5,6");
	%hash_result = FilesIO->fileToHash($resultFile, "\t", 3);

	%hash_avg = FilesIO->fileToHash($domainAvgFile, "\t", 0);
	$avg_domain = String->getElementArrayLine($hash_avg{$PFAM_ACC}, "\t", 1);
	
	print "$avg_domain\n";
	
	while ( my ($key, $value) = each(%hash_best) ) {
		
		if ($verbose eq "true"){print ">>>". $value."\n";}		
		@array_aux 	= String->StringToArray($value, "\t");
		$accModel   = $array_aux[0];
		$accNumber 	= $array_aux[3];
		$starHit	= $array_aux[5];
		$endHit 	= $array_aux[6];
		$evalue		= $array_aux[4];
		$size		= $endHit - $starHit + 1;
		$size       = $size/$avg_domain;
		if ($size > 1){ $size = 1;}
		
		
		$accNumber_clean = $accNumber;
		$new_key = $accNumber."_".$starHit."_".$endHit;
		
		$all_results	= $hash_result{$accNumber};
		@array_aux  	= String->StringToArray2d($all_results, "\n", "\t");
		@array_aux	 = Array->getSomeColsMatrix2("4:5:6:0:1:2:3:7:8:9", \@array_aux);
		@array_overlap = Blast->getHitsWithOverlap(0, $PERC_OVERLAP, "", "", $starHit, $endHit,  \@array_aux);
		if ($verbose eq "true"){print "Agree ".scalar(@array_overlap)."\n";}
		if ($verbose eq "true"){print "Total Model ".$totalModels."\n";}

		@array_models = Array->copyColMatrixToArray2(3, \@array_overlap);
		@array_models = Array->removeRepeated(\@array_models);
		
		$perc_model = scalar(@array_models)/$totalModels;
		$info = Array->printMatrix2("\t", "\n", \@array_overlap);
		if ($verbose eq "true"){print  $info."\n";}
			
		$perc_clade = getPercClade($new_key, $accNumber_clean, \@array_models);
			
			
		$info_att .= $new_key."\t". $evalue."\t". discretizeBEv($evalue,$new_key, $accNumber_clean). "\t". $size."\t".$perc_model."\t".$perc_clade."\n";
		#print $count."\n";
		$count++;
				
	}
	if ($verbose eq "true"){print  $info_att."\n";}
	if ($filtre eq "true"){
		$info_att = filtreBestAtt($info_att);	
	}
	FilesIO->saveFile($outputFile, $info_att, $verbose);
}


############################ sub main ##################################
sub filtreBestAtt{
	my $info 			= $_[0];
	
	my $smaller;			my @array_aux;
	my $new_record;			my $new_att = "";
	
	$info =~ s/_/\t/g;
	my %hash = String->stringToHash($info, "\n", "\t", 0);
	
	while ( my ($key, $value) = each(%hash) ) {
		#print "---->\n".$value."\n";
		@array_aux = String->StringToArray2d($value, "\n", "\t");
		$smaller = 0;
		if (scalar(@array_aux) > 1){
			$smaller = Array->getSmallerMatrixIndex(scalar(@array_aux), 3, \@array_aux);
		}
		if ($array_aux[$smaller][3] <= $filtreEv){
			$new_record =$array_aux[$smaller][0]."_".$array_aux[$smaller][1]."_".$array_aux[$smaller][2];
			for (my $i =3; $i <=  $#{ $array_aux[$smaller] }; $i++){
				$new_record .= "\t".$array_aux[$smaller][$i];
			} 
			#print $new_record."\n";
			$new_att .= $new_record ."\n";
		}	
	}
	return $new_att;
}
############################ sub isNumber ###################################
sub getPercClade {
 	
  	my ($accNumber, $accNumber_clean, $matrix) = @_;

	my @lmatrix = @{$matrix};
  	
  	my @array_aux = ();					my $main_clade;
  	my $protein_clades;					my $model_clades;
  	my @array_model_clades;				my $count = 0;
  	my $gaCutOFF;						my $score;
  	my $model;
  					
	$main_clade = &getMainClade($accNumber, $accNumber_clean);
	
	if ($main_clade eq ""){
		$accNumber = clearACC($accNumber);
		#$accNumber =~ s/_//g;
		$model = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_MODEL);
		if ($model eq "HMMer-3"){
			$gaCutOFF = String->getElementArrayLine($hash_avg{$PFAM_ACC}, "\t", $COL_GA);			
			$score = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_BITS_RESULT);
			if ($score >= $gaCutOFF){
				return 1;
			}else{
				print $hash_best{$accNumber}; 
				return 0;
			}  	
		}else{
			return 0;
		}			
	}
  	$model_clades = FilesIO->getLinesInFile($taxonModelFile, $main_clade, "false");
	
  	if ($model_clades eq ""){
  		return "NA";
  	}
  	
  	@array_model_clades = String->StringToArray2d($model_clades, "\n", "\t");
  

  	for (my $i = 0; $i < scalar(@lmatrix); $i++){
  		for (my $j = 0; $j < scalar(@array_model_clades); $j++){
  			if ($array_model_clades[$j][0] eq $lmatrix[$i]){
  				$count++;  				
  				$j = scalar(@array_model_clades);
  			}
  		}
  	}
  	
  	return ($count/scalar(@array_model_clades));	
  
}
############################ sub isNumber ###################################
sub getMainClade {
	my $accNumber 		= $_[0];
	my $accNumberClean 	= $_[1];
	
	my $taxonCode;				my $taxonFile;
	my $info;					my $main_clade;
	  	
	#$accNumber =  String->getElementArrayLine($accNumber, "-", 0);
 	
 	if (index($accNumberClean,"HMM") >=0){
		return "";
  	}
  	
  	if (exists($hash_taxon{$accNumberClean})){
  		#print $hash_taxon{$accNumberClean}."\n";
  		$taxonCode = String->getElementArrayLine($hash_taxon{$accNumberClean}, "\t", 1);
  		$taxonFile = $taxonDir.$taxonCode.".taxon";
  		if (FilesIO->isFile($taxonFile) eq "true"){
			$info = FilesIO->lineFileToString($taxonFile)."\n";
			$main_clade = String->getElementArrayLine($info, "\t", $level);
		}else{
			print "ERROR::".$PROG_NAME.":: $taxonFile NOT found\n";
			exit;
		}
  	}else{
  		print "ERROR::".$PROG_NAME.":: $accNumberClean does not have Taxon\n";
  	}
  	return $main_clade;
}

############################ sub isNumber ###################################
sub discretizeBEv {
	my $number 			= $_[0];
	my $accNumber 		= $_[1];
	my $accNumberClean 	= $_[2];
	
	my $gaCutOFF; 				my $score;
	my $model;					my @all_results;
	my $index;					my $evalue;
	
	if ($number <= 1e-30){
		return 1;
	}else{
		
		$accNumber = clearACC($accNumber);
		#$accNumber =~ s/_//g;
		#print "entrou 2\n";
		$gaCutOFF = String->getElementArrayLine($hash_avg{$PFAM_ACC}, "\t", $COL_GA);		
		#print $accNumber."\t".$hash_best{$accNumber}."\n";
		$score = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_BITS_RESULT);
		$model = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_MODEL);
		$evalue = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_EVALUE);
		#print "$gaCutOFF $score $model $evalue\n";
		if ($model eq "HMMer-3" && $score >= $gaCutOFF){
			return 1;
		}else{
			@all_results = String->StringToArray2d($hash_result{$accNumberClean}, "\n", "\t");
			$index = Array->isInMatrix2("HMMer-3", \@all_results);
			if ($index >=0){
				$score = $all_results[$index][$COL_BITS_RESULT];
				if ($score >= $gaCutOFF){
					return 1;
				}
			}elsif ($evalue < 1e-10){
				return 1
			}
			return 0;			
		}
	}	
}


############################ sub isNumber ###################################
sub discretizeBEv2 {
	my $number 	= $_[0];
	my $accNumber 		= $_[1];
	my $accNumberClean 	= $_[2];
	
	my $gaCutOFF; 				my $score;
	my $model;					my @all_results;
	my $index;
	
	if ($number <= 1e-30){
		return 1;
	}elsif ($number <= 1e-25){
		return 0.9;
	}elsif ($number <= 1e-20){
		return 0.8;
	}elsif ($number <= 1e-15){
		return 0.7;
	}elsif ($number <= 1e-10){
		return 0.6;
	}else{
		$accNumber = clearACC($accNumber);
		#$accNumber =~ s/_//g;
		$gaCutOFF = String->getElementArrayLine($hash_avg{$PFAM_ACC}, "\t", $COL_GA);			
		$score = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_BITS_RESULT);
		$model = String->getElementArrayLine($hash_best{$accNumber}, "\t", $COL_MODEL);
		if ($model eq "HMMer-3" && $score >= $gaCutOFF){
			return 0.6;
		}else{
			@all_results = String->StringToArray2d($hash_result{$accNumberClean}, "\n", "\t");
			$index = Array->isInMatrix2("HMMer-3", \@all_results);
			if ($index >=0){
				$score = $all_results[$index][$COL_BITS_RESULT];
				if ($score >= $gaCutOFF){
					return 0.6;
				}
			}
			if ($number <= 1e-5){
				return 0.3;
			}elsif ($number <= 1e-3){
				return 0.2;
			}elsif ($number <= 1){
				return 0.1;
			}else{
				return 0;
			}	
		}
	}	
}

sub clearACC{
		my $accNumber 	= $_[0];
		my $accNumberClean = "";
		my @array = split(/_/, $accNumber);
		$accNumberClean = $array[0];
		for (my $i=1; $i < scalar(@array)-2; $i++){
			$accNumberClean .="_".$array[$i];
		}
		$accNumberClean .= $array[scalar(@array)-2].$array[scalar(@array)-1];
		#print $accNumber."\t".$accNumberClean."\n";
	
		return $accNumberClean;
}

