#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Sequence::sequence;
use Util::Hash;


my $outputFile = "";
my $verbose    	= "false";
my $listFile = "";
my $taxonDir  = "";

my $database  = "";
my $tmpDir		= "";
my $MAX_SEQs  = 1000;
my $splitFactor = 9422;

my $PROG_NAME = "CASH_27::scripts::ensemble::createTaxonFileModels.pl";
my %hash_specie = ();


&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-listFile"){
			$listFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-taxonDir"){
			$taxonDir = $ARGV[$counter+1];
		}					
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($listFile, "ERROR::".$PROG_NAME."::Parameter -listFile having list of protein acc number is required");
	FilesIO->checkFile($listFile, "ERROR::".$PROG_NAME.":: File NOT found ".$listFile);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($taxonDir, "ERROR::".$PROG_NAME.":: Parameter -taxonDir having taxon directory is required!");
	
	print "-listFile = $listFile\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-taxonDir = $taxonDir\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $taxonCode;			my $protein;			
	my $info = "";			my $taxonFile;
	
	my @arrayList = FilesIO->fileToArray2d($listFile, "\t");
	
	for (my $i = 0; $i <scalar(@arrayList); $i++){
	
		$taxonCode =  $arrayList[$i][4];
		$taxonCode =  String->removeCaracter($taxonCode, "\n");
		$protein   =  $arrayList[$i][1];
		
		$taxonFile = $taxonDir.$taxonCode.".taxon";
		if (FilesIO->isFile($taxonFile) eq "true"){
			$info .= $protein."\t".FilesIO->lineFileToString($taxonFile)."\n";
		}else{
			print "ERROR::".$PROG_NAME.":: $taxonFile NOT found\n";
		}
	}
	
	if ($info ne ""){
		FilesIO->saveFile($outputFile, $info, $verbose);
	}
}
