#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Database::PFAM;
use Util::Array;
use Util::Hash;
use Align::Primary::blast;

my $outputFile = "";
my $verbose    	= "false";
my $resultFile  = ""; 
my $accNumberList 		= ""; 
my $evalue_CUTOFF = "1";
my $overlap = "true";
my $ignoreMinLength = "false";
my $RANGE_AA = 10;

my $PROG_NAME = "CASH_27::scripts::run::cut_domain_by_size.pl";

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-resultFile"){
			$resultFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-evalue_CUTOFF"){
			$evalue_CUTOFF = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-overlap"){
			$overlap = $ARGV[$counter+1];		
		}									
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	

	String->isEmpty($resultFile, "ERROR::".$PROG_NAME.":: Parameter -resultFile is required! ");
	FilesIO->checkFile($resultFile, "ERROR::".$PROG_NAME.":: File NOT found ".$resultFile);
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required!");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	
	print "-resultFile = $resultFile\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-evalue_CUTOFF = $evalue_CUTOFF\n";
	print "-overlap = $overlap\n";
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	
	my @array = FilesIO->fileToArray2d($resultFile, "\t");
	my $candidate;					my $starHit;
	my $endHit;						my $info = "";
	my %hash_hits = ();				my $length_domain;
	my $key;						my $accNumber;
	my @array_aux = ();				my $evalue;
	my $evalue_old;					my $getDomain;
	my $MIN_LEN_COMPARE;
 
	
	
	#print $length_domain. "\t".$min_length."\n";
	for (my $i=0; $i < scalar(@array); $i++){
		$accNumber = $array[$i][3];
		$starHit	= $array[$i][5];
		$endHit 	= $array[$i][6];
		$evalue		= $array[$i][4];
		$length_domain = $endHit - $starHit + 1;
		$key = $accNumber."_".$starHit."_".$endHit;
		#print Array->printArray("\t", $array[$i])."\n";	
		#print $length_domain. "\t".$min_length."\n";
		$getDomain = "true";		
		if ($getDomain eq "true" && $evalue <= $evalue_CUTOFF){
			#print $key."\n";
			if (exists($hash_hits{$key})){
				@array_aux = split(/\t/, $hash_hits{$key});
				$evalue_old = $array_aux[4];
				if ($evalue < $evalue_old) {
					$hash_hits{$key} =  Array->printArray("\t", $array[$i])."\n";
				}
			}else{
				$hash_hits{$key} =  Array->printArray("\t", $array[$i])."\n";
			}
			#print "saved\n";
		}	
	}	
	Hash->hashToFileOnlyValue($outputFile, \%hash_hits);
	if ($overlap eq "true"){
		$info = cutDomainByOverlap();
		FilesIO->saveFile($outputFile, $info, $verbose);
	}
}


############################ sub cutDomainByOverlap ##################################
sub cutDomainByOverlap{
	my %hash_result = FilesIO->fileToHash($outputFile, "\t", 3);
	my $info = "";				my $perc_overlap;
	my @array_prot = ();		my @array_hits = ();
	my $save;					my $AA_ALOWED = 5;
	
	while ( my ($key, $value) = each(%hash_result) ) {
		
		
		@array_prot = String->StringToArray2d($value, "\n", "\t");
		@array_prot = Array->sortArray2d(4, \@array_prot);
		#print Array->printMatrix2("\t", "\n", \@array_prot);
		@array_hits = ();
		push(@array_hits, $array_prot[0]);
		for (my $i=1; $i < scalar(@array_prot); $i++){
			#print "i= ".($i+1)."\n";
			$save = "true";
			for (my $k=0; $k < scalar(@array_hits); $k++){
				$perc_overlap = Blast->calculateOverlapSize($array_prot[$i][5], $array_prot[$i][6], $array_hits[$k][5], $array_hits[$k][6]);
				if ($perc_overlap > $AA_ALOWED){
					$save = "false";
					$k = scalar(@array_hits);
				}
			}
			if ($save eq "true"){
				push(@array_hits, $array_prot[$i]);
				#print Array->printMatrix2("\t", "\n", \@array_hits);
			}		
		}
		$info .= String->trim(Array->printMatrix2("\t", "\n", \@array_hits));			
	}	
	return $info;
}

############################ sub cutDomainByOverlap ##################################
sub cutDomainByOverlap3{
	my %hash_result = FilesIO->fileToHash($outputFile, "\t", 3);
	my %hash_aux_1;				my $starHit;
	my $endHit;					my $evalue;
	my @array_overlap;			my @array_aux_i;
	my @array_aux_j;			my $perc_overlap;
	my @arraykeys;				my $keyi;
	my $keyj;					my $info = "";
	
	while ( my ($key, $value) = each(%hash_result) ) {
	
		%hash_aux_1 = String->stringToHashConcatField($value, "\n", "\t", "3:5:6");
		@arraykeys = keys %hash_aux_1;
		for (my $i=0; $i < scalar(@arraykeys); $i++){
			$keyi = $arraykeys[$i];
			if (exists($hash_aux_1{$keyi})){
				@array_aux_i = String->StringToArray($hash_aux_1{$keyi}, "\t");
				for (my $j=$i+1; $j < scalar(@arraykeys); $j++){
					$keyj = $arraykeys[$j];
					if (exists($hash_aux_1{$keyj})){
						print " V i= ".$hash_aux_1{$keyi};
						print " V j = ".$hash_aux_1{$keyj};
					
						@array_aux_j = String->StringToArray($hash_aux_1{$keyj}, "\t");
						$perc_overlap = Blast->calculateOverlapPerc($array_aux_i[5], $array_aux_i[6], $array_aux_j[5], $array_aux_j[6]);
						if ($perc_overlap > 0){
							print " overlapAA = $perc_overlap\n";
							print "i= $array_aux_i[4] j= $array_aux_j[4]\n";	
							if ($array_aux_i[4] >= $array_aux_j[4]){
								delete($hash_aux_1{$keyi});
								$j = scalar(@arraykeys);
								print " deleted i = $keyi\n";	
							}else{
								delete($hash_aux_1{$keyj});
								print " deleted j = $keyj\n";
							}
						}
					}
				}
				#print "\n";
			}			
		}	
		$info .= Hash->hashToStringOnlyValue(\%hash_aux_1);	
	}
	#print "\n======================\n";
	return $info;
}

############################ sub cutDomainByOverlap ##################################
sub cutDomainByOverlapBKP{
	my %hash_overlap = FilesIO->fileToHash($outputFile, "\t", 3);
	my @array_aux;				my $starHit;
	my $endHit;					my $evalue;
	my @array_overlap;
	
	while ( my ($key, $value) = each(%hash_overlap) ) {
		@array_aux = String->StringToArray2d($value, "\n", "\t");
		@array_aux = Array->sortArray2d(5, \@array_aux);
		@array_aux = Array->getSomeColsMatrix2("4:5:6:0:1:2:3:7:8:9", \@array_aux);
		for (my $i=0; $i < scalar(@array_aux); $i++){
			$starHit	= $array_aux[$i][1];
			$endHit 	= $array_aux[$i][2];
			$evalue		= $array_aux[$i][0];
			@array_overlap = Blast->getHitsWithOverlap(0, 90, $starHit, $endHit, "", \@array_aux);
			print Array->printMatrix2("\t", "\n", \@array_overlap);
		}
		
	}
}
