#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
        use lib $ENV{'PERL_LIB'};
} 
else{
        print "ERROR:: Not existis PERL_LIB\n";
        exit;
}

use File::Files;
use Database::SwissProt;
use Database::PFAM;
use Util::Array;
use Util::Hash;
use Align::Primary::blast;

my $outputFile = "";
my $verbose     = "false";
my $resultFile  = ""; 
my $accNumberList               = ""; 
my $evalue_CUTOFF = "1";
my $overlap = "true";
my $ignoreMinLength = "false";
my $RANGE_AA = 10;
my $resultFileBest = "";

my $PROG_NAME = "CASH_27::scripts::run::domain_average.pl";

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
        my $counter;
        my $ending_value = scalar(@ARGV);               #numero de parametros de entrada
        for($counter = 0 ; $counter < $ending_value ; $counter++){
                if($ARGV[$counter] eq "-verbose"){
                        $verbose = $ARGV[$counter+1];
                }elsif($ARGV[$counter] eq "-outputFile"){
                        $outputFile = $ARGV[$counter+1];
                }elsif($ARGV[$counter] eq "-resultFile"){
                        $resultFile = $ARGV[$counter+1];
                }elsif($ARGV[$counter] eq "-evalue_CUTOFF"){
                        $evalue_CUTOFF = $ARGV[$counter+1];             
                }elsif($ARGV[$counter] eq "-resultFileBest"){
                        $resultFileBest = $ARGV[$counter+1];
                }                                                               
        }
}
############################ sub checkParameters ##################################
sub checkParameters {
        print "\n=============================================\n";
        print "Cheking Parameters $PROG_NAME\n";
        

        String->isEmpty($resultFile, "ERROR::".$PROG_NAME.":: Parameter -resultFile is required! ");
        FilesIO->checkFile($resultFile, "ERROR::".$PROG_NAME.":: File NOT found ".$resultFile);
        String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required!");
        String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
        String->isEmpty($resultFileBest, "ERROR::".$PROG_NAME.":: Parameter -resultFileBest having Best Result file is required!");
        FilesIO->checkFile($resultFileBest, "ERROR::".$PROG_NAME.":: File NOT found ".$resultFileBest);
        
        print "-resultFile = $resultFile\n";
        print "-outputFile = $outputFile\n";
        print "-verbose = $verbose\n";
        print "-evalue_CUTOFF = $evalue_CUTOFF\n";
        print "-overlap = $overlap\n";
        print "-resultFileBest = $resultFileBest\n";
        print "ok\n";   
}       

############################ sub main ##################################
sub main{                       
                                                
        my $all_results;                                my $PERC_OVERLAP = 0.5;
        my @array_overlap;                              my $index;
        my $startSum;                                   my $endSum;
        my $startAVG;                                   my $endAVG;
        my $count;                                      my @array_aux = ();
        my $starHitNew;                                 my $endHitNew;          
        my $size;                                       my $sizeNew;    
        my $accModel;                                   my $modelStart;
        my $endStart;                                   my $accNumber;
        my $evalue;                                     my $starHit;
        my $endHit;                                     my $score;
        my $identity;                                   my $species;
        my $info ="";   
        
        my %hash_best = FilesIO->fileToHashConcatKey($resultFileBest, "\t", "3,5,6");
        my %hash_result = FilesIO->fileToHash($resultFile, "\t", 3);

        while ( my ($key, $value) = each(%hash_best) ) {
                if ($verbose eq "true"){print ">>>". $value."\n";}      
                @array_aux      = String->StringToArray($value, "\t");
                $accModel   = $array_aux[0];
                $modelStart = $array_aux[1];
                $endStart   = $array_aux[2];
                $accNumber      = $array_aux[3];
                $evalue         = $array_aux[4];
                $starHit        = $array_aux[5];
                $endHit         = $array_aux[6];
                $score          = $array_aux[7];
                $identity       = $array_aux[8];
                $species        = $array_aux[9];
                $size           = $endHit - $starHit + 1;
                $all_results    = $hash_result{$accNumber};
                @array_aux      = String->StringToArray2d($all_results, "\n", "\t");
                @array_aux       = Array->getSomeColsMatrix2("4:5:6:0:1:2:3:7:8:9", \@array_aux);
                @array_overlap = Blast->getHitsWithOverlap(0, $PERC_OVERLAP, "", "", $starHit, $endHit,  \@array_aux);
             
                @array_overlap = Blast->removeWeakOverlap(0, $PERC_OVERLAP, "", "", $starHit, $endHit,  \@array_overlap);
                @array_overlap = Blast->removeHitsEvalue($evalue_CUTOFF, \@array_overlap);
                #print Array->printMatrix2("\t", "\n", \@array_overlap);
        
                @array_overlap =  Array->sortArray2d(0, \@array_overlap);
                @array_aux = Array->copyColMatrixToArray2(0, \@array_overlap);
                $index = Array->getIndexMediana(\@array_aux);
                $startSum = 0;
                $endSum = 0;
                $count = 0;
                for (my $i=0; $i<= $index; $i++){
                        $startSum += $array_overlap[$i][1];
                        $endSum   += $array_overlap[$i][2];
                        $count++;
                }
                $starHitNew = int($startSum/$count);
                $endHitNew = int($endSum/$count);
                $sizeNew                = $endHitNew - $starHitNew + 1;
                $info .= $accModel."\t".$modelStart."\t".$endStart."\t".$accNumber."\t".$evalue."\t";
                if ($sizeNew < $size){
                        $info .= $starHitNew."\t".$endHitNew."\t";
                }else{
                        $info .= $starHit."\t".$endHit."\t";
                }
                $info .= $score."\t".$identity."\t".$species."\n";
                #print "$info\n";
                
        }
        FilesIO->saveFile($outputFile, $info, $verbose);
        
}