#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Sequence::sequence;
use Sequence::protein;
use Util::Hash;
use HMMs::HMMER::hmmer;


my $outputFile = "";
my $verbose    	= "false";
my $mutation = "0.2";
my $listFile  = "";
my $sequenceFile = "";
my $numSeq = "1000";
my $tmpDir =  "";
my $hmmFile = "";

my $PROG_NAME = "CASH_27::scripts::ensemble::generate_training_sequences.pl";


	

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-listFile"){
			$listFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-mutation"){
			$mutation = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-sequenceFile"){
			$sequenceFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-numSeq"){
			$numSeq = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-hmmFile"){
			$hmmFile = $ARGV[$counter+1];
		}				
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($listFile, "ERROR::".$PROG_NAME."::Parameter -listFile having list of protein acc number is required");
	FilesIO->checkFile($listFile, "ERROR::".$PROG_NAME.":: File NOT found ".$listFile);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($sequenceFile, "ERROR::".$PROG_NAME."::Parameter -sequenceFile having sequence file in FASTA format is required");
	FilesIO->checkFile($sequenceFile, "ERROR::".$PROG_NAME.":: File NOT found ".$sequenceFile);	
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME."::Parameter -tmpDir having temporary directory is required");
	String->isEmpty($hmmFile, "ERROR::".$PROG_NAME."::Parameter -hmmFile having HMM file is required");
	FilesIO->checkFile($hmmFile, "ERROR::".$PROG_NAME.":: File NOT found ".$hmmFile);	
	
	print "-listFile = $listFile\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-mutation = $mutation\n";
	print "-sequenceFile = $sequenceFile\n";
	print "-hmmFile = $hmmFile\n";	
	print "-numSeq = $numSeq\n";
		
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $accNumber;				my $ann;
	my $seq;					my $interval;
	my @array_range;			my $start;
	my $end;					my $sequence;
	my $count = 1;				my $info = "";
	my $new_ann;				my $total_seq;
	my @array_seq_hmm = ();
		
	my %hash = FilesIO->fileToHash($listFile, "\t", 1);
	my @array_seq = Sequences->getSequenceFromFastaArray2D($sequenceFile); 
	
	$total_seq = $numSeq - scalar(@array_seq);
	
	while ($count <= $total_seq){
		HMMER->createRandomSequences($hmmFile, " -N 1 ", $tmpDir.$count.".fasta", $verbose);
		@array_seq_hmm = Sequences->getSequenceFromFastaArray2D($tmpDir.$count.".fasta");
		$ann = ">HMM-sample-$count\n";
		$seq = $array_seq_hmm[0][1];
		$info .= $ann.uc(Sequences->divideSeq($seq, 60))."\n";
		$count++;
		for (my $i=0; $i < scalar(@array_seq) && $count <= $total_seq; $i++){
			$ann = $array_seq[$i][0];
			$seq = $array_seq[$i][1];
		
			$accNumber = String->getElementArrayLine($ann, "\t", 0);
			$accNumber =  String->removeCaracter($accNumber, ">");
			if (exists($hash{$accNumber})){
				$interval = String->getElementArrayLine($hash{$accNumber}, "\t", 3);
				@array_range = split(/-/,$interval);
				$start 	= $array_range[0];
				$end 	= $array_range[1];
				$sequence = String->removeCaracter($array_seq[$i][1], "\n");
				$sequence = Protein->mutateProtein($sequence, $mutation, $start, $end);
				$ann  =~ s/$accNumber/$accNumber-sample$count/g;
				$info .= $ann.uc(Sequences->divideSeq($sequence, 60))."\n";
				$count++;
			}else{
				print "ERROR::".$PROG_NAME."::$accNumber NOT found in $listFile\n";
				exit;
			}	
		}
		
	}
	if ($info ne ""){
		FilesIO->saveFile($outputFile, $info, $verbose);
	}	
}
	