#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Sequence::sequence;
use Util::Hash;


my $outputFile = "";
my $verbose    	= "false";
my $accNumberList = "";
my $databaseList  = "";

my $database  = "";
my $tmpDir		= "";
my $MAX_SEQs  = 1000;
my $splitFactor = 9422;

my $PROG_NAME = "CASH_27::scripts::ensemble::get_training_sequences.pl";
my %hash_specie = ();


&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-accNumberList"){
			$accNumberList = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-database"){
			$database = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-MAX_SEQs"){
			$MAX_SEQs = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-splitFactor"){
			$splitFactor = $ARGV[$counter+1];
		}						
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($accNumberList, "ERROR::".$PROG_NAME."::Parameter -accNumberList having list of protein acc number is required");
	FilesIO->checkFile($accNumberList, "ERROR::".$PROG_NAME.":: File NOT found ".$accNumberList);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having temporary directory is required!");
	String->isEmpty($database, "ERROR::".$PROG_NAME.":: Parameter -database has index database is required");
	
	print "-accNumberList = $accNumberList\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-tmpDir = $tmpDir\n";
	print "-database = $database\n";
	print "-MAX_SEQs = $MAX_SEQs\n";
	print "-splitFactor = $splitFactor\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	
	my %hash = FilesIO->fileToHash($accNumberList, "\t", 1);
	my %record_sequence;			my $j=0;
	my $sequences = "";				my @array_acc = ();			
	my %hash_species = ();
	
	@array_acc = keys %hash;
	while ( $j < scalar(@array_acc)) {
		if ($verbose eq "true"){print "DEBUG ".$array_acc[$j]."\n";}
		$sequences .= getSequence($array_acc[$j]);
		if ($j == ($MAX_SEQs-1)){
			$j = scalar(@array_acc);
		}
		$j++;
	}
	
	if ($sequences ne ""){
		FilesIO->saveFile($outputFile, $sequences, $verbose);
	}
}
		
############################ sub main ##################################
sub getSequence{
	my $accNumber	= $_[0];
	
	my $info = "";			my @array_aux = ();
	my $line;				my $numberFile;
	my $position;			my $new_db;
	my $tmpFile = $tmpDir."tmp.txt";
	
	#FilesIO->removeFile($tmpFile, $verbose);
	my $cmd = "grep -m 1 '\^>$accNumber' $database.list > $tmpFile";
	if ($verbose eq "true"){print $cmd."\n";}
	system($cmd);
	
	FilesIO->checkFile($tmpFile, "ERROR::".$PROG_NAME."::  $accNumber NOT found in $database.list ".$tmpFile);	
	$info = FilesIO->lineFileToString($tmpFile);
	@array_aux = split(/:/, $info);
	if (scalar(@array_aux) > 2){
		print "ERROR::".$PROG_NAME.":: $accNumber duplicated in $database.list";
		exit;
	}
	$line =String->removeCaracter($array_aux[1], "\n"); 
	if ($verbose eq "true"){print $line."\n";}
	#FilesIO->removeFile($tmpFile, $verbose);
	#$cmd = " head -n $line $database | tail -n 1 > $tmpFile";
	$numberFile = int($line/$splitFactor);
	$position = $line % $splitFactor;	
	if ($position == 0){
		$position = $splitFactor;
		$numberFile--;
	}
	$new_db = $database.".".$numberFile;
	$cmd = "awk \"NR\=\=$position\{print;exit\}\" $new_db > $tmpFile";
	#$cmd = $ENV{'PERL_LIB'}."/run/seek.bin $database $line > $tmpFile";
	if ($verbose eq "true"){print $cmd."\n";}
	system($cmd);
	FilesIO->checkFile($tmpFile, "ERROR::".$PROG_NAME.":: $accNumber NOT found in $new_db ($line, $position)  ".$tmpFile);
	$info = FilesIO->lineFileToString($tmpFile);	
	@array_aux = split(/:/, $info);
	if (index($array_aux[0], $accNumber) < 0){
		print "ERROR::".$PROG_NAME.":: $accNumber NOT in $array_aux[0]";
		exit
	}
	$info = $array_aux[0]."\n".Sequences->divideSeq($array_aux[1], 60);
	
	#print "DEBUG ".$info."\n";
	return $info;
}
