#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use SVM::libsvm;
use SVM::svmlight;
use Util::Number;
use Util::Array;
use SVM::svm;

my $PROG_NAME = "CASH_27::scripts::ensemble::predict_svm.pl";


my $svmModel = "";
my $attributesFileTest = "";
my $outputFile = "";
my $verbose    	= "false";
my $tmpDir = "";
my $inverseCols = "1";
my $logCols = "7";

my $POS_CLASS = 1;
my $NEG_CLASS = -1;
my $MIN_POS = 6;
my $NUMBER_ATT = 5;

my $parametersFileTrain = "";

&getParameters;
&checkParameters;


my @array_inverse_cols = split(/:/, $inverseCols);
my @array_log_cols = split(/:/, $logCols);
&main;
#&mainGist;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-svmModel"){
			$svmModel = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-attributesFileTest"){
			$attributesFileTest = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inverseCols"){
			$inverseCols = $ARGV[$counter+1];
		}										
	}
}
############################ sub checkParameters ##################################
sub checkParameters {

	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";

	String->isEmpty($svmModel, "ERROR::".$PROG_NAME.":: Parameter -svmModel is required!");
	FilesIO->checkFile($svmModel, "ERROR::".$PROG_NAME.":: File NOT found ".$svmModel);
	
	String->isEmpty($attributesFileTest, "ERROR::".$PROG_NAME.":: Parameter -attributesFileTest is required!");
	FilesIO->checkFile($attributesFileTest, "ERROR::".$PROG_NAME.":: File NOT found ".$attributesFileTest);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required!");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false! ");
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having temporary directory is required!");
	
	print "-attributesFileTest = $attributesFileTest\n";
	print "-svmModel = $svmModel\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-tmpDir = $tmpDir\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $train_file_pos = $tmpDir."train_pos.txt";
	my $train_file_neg = $tmpDir."train_neg.txt";
	my $train_file = $tmpDir."train.txt";
	my $test_file = $tmpDir."test.txt";
	my $train_model_file = $tmpDir."train.model";
	my $classOutputTest = $tmpDir."class.test.output";
	my $classOutputTrain = $tmpDir."class.train.output";
	my $confValueOutputTest= $tmpDir."conf_value.test.output";
	my $confValueOutputTrain= $tmpDir."conf_value.train.output";
	my $testResult = $tmpDir."test.result";
	my $trainResult = $tmpDir."train.result";
	my $confValueAllOutput= $tmpDir."conf_value.all.output";
	my $info = "";
	
	

	setClass($attributesFileTest, $test_file, 1, $NUMBER_ATT);
	&normalize($test_file, $test_file, 1, 0);
	
	my $info_test = LIBSVM->createDataset($test_file);
	FilesIO->saveFile($test_file, $info_test, $verbose);

	LIBSVM->predict($test_file, $svmModel, $classOutputTest, $confValueOutputTest, " ", $verbose);	

	#getResults
	my @arrar_par_test = FilesIO->fileToArray2d($attributesFileTest, "\t", " ");
	#print Array->printMatrix2("\t", "\n", \@arrar_par_test);
	my @array_aux_test = Array->copyColMatrixToArray(scalar(@arrar_par_test), $#{$arrar_par_test[0]} + 1, 0, \@arrar_par_test);
	
	
	LIBSVM->getResultsProbability($classOutputTest, $outputFile.".prob", 2, \@array_aux_test);
	

	if (FilesIO->isFile($outputFile.".prob") eq "false"){
	#if (FilesIO->isFile($outputFile) eq "false"){
		print "ERROR::".$PROG_NAME.":: File ".$outputFile.".prob NOT created\n";
	}else{
		print "LOG Criado::".$PROG_NAME.":: File ".$outputFile.".prob created\n";
	}
}

############################ sub main ##################################
sub setClass{
	my $train_file 	= $_[0];	
	my $out_file 	= $_[1];	
	my $class 		= $_[2];
	my $numberAtt	= $_[3];
	
	my $info = "";
	
	my @arrar_par_train = FilesIO->fileToArray2d($train_file, "\t", " ");
	my $cols = $#{$arrar_par_train[0]} + 1;
	if ($cols < $numberAtt){
		print "ERROR::".$PROG_NAME.":: File $train_file there are not enougth parameters\n";
		exit;
	}
	for (my $i=0; $i < scalar(@arrar_par_train); $i++){
		for (my $j=0; $j < $cols; $j++){
			$info .=$arrar_par_train[$i][$j]."\t";
		}
		$info .="$class\n";
	}	
	FilesIO->saveFile($out_file, $info, $verbose);	
}

############################ sub main ##################################
sub normalize{

		my $test_file_in 	= $_[0];
		my $test_file_out 	= $_[1];
		my $upper		 	= $_[2];
		my $bound		 	= $_[3];
				
		my @array_aux;				my $info_test = "";
		my $info_train = "";		
		
		my @arrar_par_test = FilesIO->fileToArray2d($test_file_in, "\t");
	
		my @array = @arrar_par_test;
		my $cols = $#{$array[0]};		
		
		for (my $i=1; $i < $cols; $i++){
			if (Array->isInArray($i, \@array_inverse_cols) eq "true"){
				@array 	= Array->logMatrizByCol(10, $i, \@array);
				@array =  Array->multiplyValueMatrizByCol((-1), 1, \@array);
			}				
		}
		
		@array = Array->normalizeByColunsMinMax($bound, $upper, 0, 200, "", "0:2:3:4:5:6", \@array);
		@array = Array->removeColsMatriz(0, \@array);
		$info_test = Array->printMatrix2("\t", "\n", \@array);
		FilesIO->saveFile($test_file_out, $info_test, $verbose);
		
}