#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Sequence::sequence;
use Sequence::protein;
use Util::Hash;

my $outputFile = "";
my $verbose    	= "false";
my $listFile  = "";
my $sequenceFile = "";


my $PROG_NAME = "CASH_27::scripts::ensemble::put_taxonCode_negative.pl";


	

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-listFile"){
			$listFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-sequenceFile"){
			$sequenceFile = $ARGV[$counter+1];
		}				
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($listFile, "ERROR::".$PROG_NAME."::Parameter -listFile having list of protein acc number is required");
	FilesIO->checkFile($listFile, "ERROR::".$PROG_NAME.":: File NOT found ".$listFile);	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($sequenceFile, "ERROR::".$PROG_NAME."::Parameter -sequenceFile having sequence file in FASTA format is required");
	FilesIO->checkFile($sequenceFile, "ERROR::".$PROG_NAME.":: File NOT found ".$sequenceFile);	
	
	
	print "-listFile = $listFile\n";
	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-sequenceFile = $sequenceFile\n";
		
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $accNumber;				my $ann;
	my $seq;					my $interval;
	my @array_range;			my $start;
	my $end;					my $sequence;
	my $count = 1;				my $info = "";
	my $new_ann;				my $taxonCode;
	my $rand;
		
	my @array_taxon = FilesIO->lineFileToArray($listFile);
	my @array_seq = Sequences->getSequenceFromFastaArray2D($sequenceFile); 
	
	for (my $i=0; $i < scalar(@array_seq); $i++){
		$ann = $array_seq[$i][0];
		$seq = $array_seq[$i][1];
		
		$accNumber = String->getElementArrayLine($ann, "\t", 0);
		$rand = int(rand(@array_taxon));
		$taxonCode = $array_taxon[$rand];
		$taxonCode = String->removeCaracter($taxonCode, "\n");
		$new_ann = $accNumber."\t".$taxonCode."\n";
		$info .= $new_ann.uc(Sequences->divideSeq($seq, 60))."\n";
		
	}
	if ($info ne ""){
		FilesIO->saveFile($outputFile, $info, $verbose);
	}	
}
	