#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use SVM::libsvm;
use SVM::svmlight;
use Util::Number;
use Util::Array;
use SVM::svm;

my $PROG_NAME = "CASH_27::scripts::ensemble::svm_cutOFF.pl";


my $inputFileNeg = "";
my $inputFilePos = "";
my $outputFile = "";
my $fator = "2";
my $PFAM_ACC = "";
my $negative = "true";
my $verbose    	= "false";

#my @array =(); $array[0] = 1;$array[1] = 2;$array[2] = 4;
#print Array->getHarmonicMean(\@array); exit;

&getParameters;
&checkParameters;


&main;
#&mainGist;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inputFileNeg"){
			$inputFileNeg = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inputFilePos"){
			$inputFilePos = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-factor"){
			$fator = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-PFAM_ACC"){
			$PFAM_ACC = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-negative"){
			$negative = $ARGV[$counter+1];		
		}						
	}
}
############################ sub checkParameters ##################################
sub checkParameters {

	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";

	String->isEmpty($inputFileNeg, "ERROR::".$PROG_NAME.":: Parameter -inputFileNeg is required!");
	FilesIO->checkFile($inputFileNeg, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFileNeg);
	String->isEmpty($inputFilePos, "ERROR::".$PROG_NAME.":: Parameter -inputFilePos is required!");
	FilesIO->checkFile($inputFilePos, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFilePos);
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required!");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false! ");
	String->isEmpty($PFAM_ACC, "ERROR::".$PROG_NAME.":: Parameter -PFAM_ACC having PFAM accession number is required!");
	
	print "-inputFilePos = $inputFilePos\n";
	print "-inputFileNeg = $inputFileNeg\n";	
	print "-outputFile = $outputFile\n";
	print "-fator = $fator\n";
	print "-PFAM_ACC = $PFAM_ACC\n";
	print "-negative = $negative\n";
	print "-verbose = $verbose\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $cutOFF;
	my @dataNeg = FilesIO->fileToArray2d($inputFileNeg, "\t");
	my @dataPos = FilesIO->fileToArray2d($inputFilePos, "\t");
	
	my @arrayScoresNeg = Array->copyColMatrixToArray2(1, \@dataNeg);
	my @arrayScoresPos = Array->copyColMatrixToArray2(1, \@dataPos);
	
	#my $mean = Array->getAverage(\@arrayScoresNeg);
	my $mean = Array->getHarmonicMean(\@arrayScoresNeg);
	my $std =  Array->getSTD(\@arrayScoresNeg);
	
	my $negCutOff = $mean + ($fator*$std);
	my $posCutOff = Array->getSmaller(\@arrayScoresPos);
	#print Array->printArray("\n", \@arrayScoresPos);
	#$negCutOff = Array->getBigger(\@arrayScoresNeg);
	
	$mean = Array->getHarmonicMean(\@arrayScoresPos);
	$std =  Array->getSTD(\@arrayScoresPos);
	$posCutOff = $mean - ($fator*$std);

	print "LOG::PC=".$posCutOff."\t"."NC=".$negCutOff."\n";
	if ($negative eq "true"){
		$cutOFF = $negCutOff; 
	}else{
		$cutOFF = $posCutOff; 
	}
		
	#}
	
	my $info = $PFAM_ACC."\t".$cutOFF."\n";
	FilesIO->saveFile($outputFile, $info, $verbose);
	
}

