#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use SVM::libsvm;
use SVM::svmlight;
use Util::Number;
use Util::Array;
use SVM::svm;

my $PROG_NAME = "CASH_27::scripts::ensemble::svr_dist.pl";


my $inputFileNeg = "";
my $inputFilePos = "";
my $outputFile = "";
my $fator = "2";
my $PFAM_ACC = "";
my $verbose    	= "false";


&getParameters;
&checkParameters;


&main;
#&mainGist;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inputFileNeg"){
			$inputFileNeg = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inputFilePos"){
			$inputFilePos = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-PFAM_ACC"){
			$PFAM_ACC = $ARGV[$counter+1];		
		}						
	}
}
############################ sub checkParameters ##################################
sub checkParameters {

	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";

	String->isEmpty($inputFileNeg, "ERROR::".$PROG_NAME.":: Parameter -inputFileNeg is required!");
	FilesIO->checkFile($inputFileNeg, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFileNeg);
	String->isEmpty($inputFilePos, "ERROR::".$PROG_NAME.":: Parameter -inputFilePos is required!");
	FilesIO->checkFile($inputFilePos, "ERROR::".$PROG_NAME.":: File NOT found ".$inputFilePos);
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile is required!");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false! ");
	String->isEmpty($PFAM_ACC, "ERROR::".$PROG_NAME.":: Parameter -PFAM_ACC having PFAM accession number is required!");
	
	print "-inputFilePos = $inputFilePos\n";
	print "-inputFileNeg = $inputFileNeg\n";	
	print "-outputFile = $outputFile\n";
	print "-fator = $fator\n";
	print "-PFAM_ACC = $PFAM_ACC\n";
	print "-verbose = $verbose\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $cutOFF;				my $count_missing =0; 		my $count_sample = 0;
	my $perc;
	my @dataNeg = FilesIO->fileToArray2d($inputFileNeg, "\t");
	my @dataPos = FilesIO->fileToArray2d($inputFilePos, "\t");
	
	my @arrayScoresNeg = Array->copyColMatrixToArray2(1, \@dataNeg);
	my @arrayScoresPos = Array->copyColMatrixToArray2(1, \@dataPos);
	
	my $negCutOff = Array->getBigger(\@arrayScoresNeg);
	print "negCutOff=$negCutOff\n";
	
	$cutOFF = Array->countGreaterSmallerThan("smaller", $negCutOff, \@arrayScoresPos);
	for (my $i=0; $i<scalar(@dataPos); $i++){
		if ($dataPos[$i][1] < $negCutOff){
			$count_missing++;
			if (index($dataPos[$i][0], "sample") >=0){
				$count_sample++;
			}
		}
	}
	if ($count_missing > 0){
		$perc = $count_sample/$count_missing;
		my $info = $PFAM_ACC."\t".$perc."\n";
		FilesIO->saveFile($outputFile, $info, $verbose);
	}
	
	
}

