#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use SVM::libsvm;
use SVM::svmlight;
use Util::Number;
use Util::Array;
use SVM::svm;

my $PROG_NAME = "CASH_27::scripts::ensemble::run_svm.pl";


my $attributesFilePosTrain = "";
my $attributesFileNegTrain = "";


my $outputFile = "";
my $verbose    	= "false";
my $tmpDir = "";
#my $inverseCols = "0:1:2:3:7:8:9";
#my $inverseCols = "0:1:2:3:4:5:6:7:8:9";
#my $inverseCols = "1:3";
my $inverseCols = "1";
my $logCols = "7";

my $POS_CLASS = 1;
my $NEG_CLASS = -1;
my $MIN_POS = 6;
my $NUMBER_ATT = 5;

my $parametersFileTrain = "";

&getParameters;
&checkParameters;


my @array_inverse_cols = split(/:/, $inverseCols);
my @array_log_cols = split(/:/, $logCols);
&main;
#&mainGist;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-attributesFilePosTrain"){
			$attributesFilePosTrain = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-attributesFileNegTrain"){
			$attributesFileNegTrain = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-inverseCols"){
			$inverseCols = $ARGV[$counter+1];
		}										
	}
}
############################ sub checkParameters ##################################
sub checkParameters {

	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";

	String->isEmpty($attributesFilePosTrain, "ERROR::".$PROG_NAME.":: Parameter -attributesFilePosTrain is required!");
	FilesIO->checkFile($attributesFilePosTrain, "ERROR::".$PROG_NAME.":: File NOT found ".$attributesFilePosTrain);
	String->isEmpty($attributesFileNegTrain, "ERROR::".$PROG_NAME.":: Parameter -attributesFileNegTrain is required!");
	FilesIO->checkFile($attributesFileNegTrain, "ERROR::".$PROG_NAME.":: File NOT found ".$attributesFileNegTrain);	
		


	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false! ");
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having temporary directory is required!");
	
	print "-attributesFilePosTrain = $attributesFilePosTrain\n";
	print "-attributesFileNegTrain = $attributesFileNegTrain\n";


	print "-outputFile = $outputFile\n";
	print "-verbose = $verbose\n";
	print "-tmpDir = $tmpDir\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $train_file_pos = $tmpDir."train_pos.txt";
	my $train_file_neg = $tmpDir."train_neg.txt";
	my $train_file = $tmpDir."train.txt";

	my $train_model_file = $tmpDir."train.model";
	my $classOutputTrain = $tmpDir."class.train.output";
	
	my $confValueOutputTrain= $tmpDir."conf_value.train.output";

	my $trainResult = $tmpDir."train.result";
	my $confValueAllOutput= $tmpDir."conf_value.all.output";
	my $info = "";
	
	
	#balance($attributesFilePosTrain, $attributesFileNegTrain,  $train_file_neg, $train_file_pos);
	setClass($attributesFilePosTrain, $train_file_pos, $POS_CLASS, $NUMBER_ATT);
	setClass($attributesFileNegTrain, $train_file_neg, $NEG_CLASS, $NUMBER_ATT);	

	
	$parametersFileTrain = $tmpDir."pos.neg.train.txt";
	FilesIO->concat($train_file_pos, $train_file_neg, $parametersFileTrain);
	LIBSVM->balanceDataWithReplace($POS_CLASS, $NEG_CLASS, $parametersFileTrain, $parametersFileTrain);
	
	
	&normalize($parametersFileTrain, $train_file, 1, 0);
	
	my $info_train = LIBSVM->createDataset($train_file);

	
	FilesIO->saveFile($train_file, $info_train, $verbose);
	

	LIBSVM->train($train_file, "0", "3", $train_model_file, " ", $verbose);
	
	FilesIO->copyFile($train_model_file, $outputFile, $verbose);


}


############################ sub loadAlign1d ####################################
sub normalizeByColuns{
	my ($bound, $upper, $MinXo, $MaxXo, $rowsIgnore, $colsIgnore, $matrix) = @_;
	
	my @localmatrix = @{$matrix};
	

	my @array_aux;
	
	my $delta = $upper - $bound;	
	my $deltaxo;	
	my @array_rows_ignore = split(/:/, $rowsIgnore);
	my @array_cols_ignore = split(/:/, $colsIgnore);
	
	my $rows = scalar(@localmatrix);
	my $cols =  $#{$localmatrix[0]} + 1;
	
	my @array_aux_local = Array->removeRowsMatriz($rowsIgnore, \@localmatrix);
	
	for (my $j = 0; $j < $cols; $j++){
		if (Array->isInArray($j, \@array_cols_ignore) eq "false"){
			@array_aux = Array->copyColMatrixToArray($rows-(scalar(@array_rows_ignore)), $cols, $j, \@array_aux_local);
			$deltaxo = $MaxXo - $MinXo;
			#print "MaxXo=$MaxXo MinXo=$MinXo deltaxo=$deltaxo\n";
			for (my $i = 0; $i <scalar(@localmatrix); $i++){#i = row
				if (Array->isInArray($i, \@array_rows_ignore) eq "false"){
					$localmatrix[$i][$j] = $bound + (($localmatrix[$i][$j] - $MinXo)*$delta/$deltaxo);
					if ($localmatrix[$i][$j] < $bound){
						$localmatrix[$i][$j]  = $bound;						
					}elsif($localmatrix[$i][$j] > $upper){
						$localmatrix[$i][$j] = $upper;
					}
				}
			}
		}
	}
	return @localmatrix;
}

############################ sub main ##################################
sub setClass{
	my $train_file 	= $_[0];	
	my $out_file 	= $_[1];	
	my $class 		= $_[2];
	my $numberAtt	= $_[3];
	
	my $info = "";
	
	my @arrar_par_train = FilesIO->fileToArray2d($train_file, "\t", " ");
	my $cols = $#{$arrar_par_train[0]} + 1;
	if ($cols < $numberAtt){
		print "ERROR::".$PROG_NAME.":: File $train_file there are not enougth parameters\n";
		exit;
	}
	for (my $i=0; $i < scalar(@arrar_par_train); $i++){
		for (my $j=0; $j < $cols; $j++){
			$info .=$arrar_par_train[$i][$j]."\t";
		}
		$info .="$class\n";
	}	
	FilesIO->saveFile($out_file, $info, $verbose);	
}

############################ sub main ##################################
sub normalize{
		my $train_file_in 	= $_[0];	
		my $train_file_out 	= $_[1];	
		my $upper		 	= $_[2];
		my $bound		 	= $_[3];
			
		my @array_aux;				my $info_test = "";
		my $info_train = "";		my $Min_value;
		my $Max_value;
				
		my @arrar_par_train = FilesIO->fileToArray2d($train_file_in, "\t");
	
		my @array = @arrar_par_train;
		my $cols = $#{$array[0]};		
		
		
		for (my $i=1; $i < $cols; $i++){
			if (Array->isInArray($i, \@array_inverse_cols) eq "true"){
				@array 	= Array->logMatrizByCol(10, $i, \@array);
				@array =  Array->multiplyValueMatrizByCol((-1), 1, \@array);
			}				
		}
		@array = Array->normalizeByColunsMinMax($bound, $upper, 0, 200, "", "0:2:3:4:5:6", \@array);
		@array = Array->removeColsMatriz(0, \@array);
		$info_train =  Array->printMatrix2("\t", "\n", \@array);
		
	
		FilesIO->saveFile($train_file_out, $info_train, $verbose);
	#	print $train_file_out."\n";
	

	
	
}