#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Sequence::sequence;
use Database::PFAM;

my $listFile = "";

my $verbose    	= "false";
my $outputDir = "";
my $taxonFiles = "";
my $dirNewModels = "";
my $dirOldModels = "";
my $tmpDir = "";



my $k = 2; #factor of split
my $ext = "txt";

my $PROG_NAME = "CASH_27::scripts::prepare::organiseModels.pl";


&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-listFile"){
			$listFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputDir"){
			$outputDir= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-dirNewModels"){
			$dirNewModels = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-dirOldModels"){
			$dirOldModels = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}								
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	String->isEmpty($listFile, $PROG_NAME."::ERROR::Parameter -listFile having Pfam aligments is required!");
	FilesIO->checkFile($listFile, $PROG_NAME."ERROR:: File $listFile NOT found ");	
	String->isEmpty($outputDir, $PROG_NAME."::ERROR::Parameter -outputDir is required!");
	String->isEmpty($dirNewModels, $PROG_NAME."::ERROR::Parameter -dirNewModels is required!");
	String->isEmpty($dirOldModels, $PROG_NAME."::ERROR::Parameter -dirOldModels is required!");
	String->isEmpty($tmpDir, $PROG_NAME."::ERROR::Parameter -tmpDir is required!");

	print "-listFile = $listFile\n";
	print "-outputDir = $outputDir\n";
	print "-dirNewModels = $dirNewModels\n";
	print "-dirOldModels = $dirOldModels\n";
	print "-verbose = $verbose\n";
	print "-tmpDir=$tmpDir\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	
	my $accPfam;			my $accProt;
	my $accSpecie;			my $interval;
	my $nameFileNew;		my $nameFileOld;	
	my $nameFileTmpGZ;		my $nameFileTmp;
	my $countNF = 0;		my $info = "";
	my @newFile;			my $nameFile;
	my @array_aux;			my $codeTaxon;

	
	my @array = FilesIO->fileToArray2d($listFile, "\t");
	
	for ( my $i=0; $i < scalar(@array); $i++){
		$accPfam	= $array[$i][0];
		$accProt 	= $array[$i][1];
		$accSpecie 	= $array[$i][2];
		$interval 	= $array[$i][3];
		$codeTaxon 	= $array[$i][4];
		
		$nameFileNew = $dirNewModels."/".$accPfam."/".$accProt."_".$accSpecie."_".$interval.".chk";
		$nameFileOld = $dirOldModels."/".$accPfam."/".$accProt."_".$accSpecie."_".$interval.".chk";
		$nameFileTmp = $tmpDir."/".$accPfam."/".$accProt."_".$accSpecie."_".$interval.".chk";
		$nameFileTmpGZ = $tmpDir."/".$accPfam."/".$accProt."_".$accSpecie."_".$interval.".chk.tar.gz";
		
		
		if (FilesIO->isFile($nameFileNew) eq "true"){
			FilesIO->copyFileDir($nameFileNew, $tmpDir."/".$accPfam."/", $verbose);
		}elsif (FilesIO->isFile($nameFileOld) eq "true"){
			FilesIO->copyFileDir($nameFileOld, $tmpDir."/".$accPfam."/", $verbose);
		}elsif (FilesIO->isFile($nameFileNew.".tar.gz") eq "true"){
			copyUntar($nameFileNew.".tar.gz", $tmpDir."/".$accPfam."/");
		}elsif (FilesIO->isFile($nameFileOld.".tar.gz") eq "true"){
			copyUntar($nameFileOld.".tar.gz", $tmpDir."/".$accPfam."/");
		}
		if (FilesIO->isFile($nameFileTmp) eq "true"){
			$info .= Array->copyRowMatrixToString("\t", $i, \@array)."\n";
		}else{
			@newFile = FilesIO->listDirectoryFromPrefixFile($dirOldModels."/".$accPfam."/".$accProt."*", $tmpDir, $verbose);
			if (scalar(@newFile) > 0){
				$nameFile = FilesIO->getNameFile($newFile[0]);
				@array_aux = split(/\./, $nameFile);
				@array_aux = split(/_/, $array_aux[0]);
				$accSpecie = $array_aux[1];
				$interval  = $array_aux[2];
				$info .= $accPfam."\t".$accProt."\t".$accSpecie."\t".$interval."\t".$codeTaxon."\n";
				copyUntar($newFile[0], $tmpDir."/".$accPfam."/");				
			}else{
				print "LOG::".$nameFileNew ." NOT found\n";
				$countNF++;
			}
		}		
	}
	print "LOG:: Not created $countNF\n";
	
	if (FilesIO->isDirectory($tmpDir."/".$accPfam."/") eq "true"){
		FilesIO->compactTgz($tmpDir."/".$accPfam."/", $verbose);
		FilesIO->copyFileDir($tmpDir."/".$accPfam.".tar.gz", $outputDir, $verbose);
		FilesIO->saveFile($listFile.".final", $info, $verbose);
		FilesIO->removeFile($tmpDir."/".$accPfam.".tar.gz", $verbose);
		FilesIO->removeDirectory($tmpDir."/".$accPfam."/", $verbose);
	}else{
		print "ERROR:: Dir NOT exixts:".$tmpDir."/".$accPfam."/";
	}
}

############################ sub main ##################################
sub copyUntar{
	my $fileTar 	= $_[0];
	my $tmpDirLocal = $_[1];
	my $nameFile =  FilesIO->getNameFile($fileTar);
	my $nameFileTmp = $tmpDirLocal.$nameFile;
	
	FilesIO->copyFileDir($fileTar, $tmpDirLocal, $verbose);
	if (index($nameFileTmp, ".gz") >=0){
		FilesIO->uncompactTgz($nameFileTmp);
		FilesIO->removeFile($nameFileTmp, $verbose);
	}
	
	
}

	