#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::ScopPattern;


my $fileTestSet	= "";
my $virtualPath = "";
my $outputFileList  = "";
my $exceptSupFamily			= "";
my $kindRun 		= "";
my $defaultFamily	= "";
my $verbose			= "false";

my $PROG_NAME = "CASH_27::scripts::prepare::prepareListDataSet.pl";	

&getParameters;
&checkParameters;
&main;

############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-fileTestSet"){
			$fileTestSet = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-virtualPath"){
			$virtualPath= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-outputFileList"){
			$outputFileList= $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];		
		}
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($fileTestSet, "ERROR::".$PROG_NAME.":: Parameter -fileTestSet having files to compare is required! ");
	FilesIO->checkFile($fileTestSet, "ERROR::".$PROG_NAME.":: FILE NOT found ".$fileTestSet);	
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	
	String->isEmpty($virtualPath, "ERROR::".$PROG_NAME.":: Parameter -virtualPath having virtual path of list files is required!");
	String->isEmpty($outputFileList, "ERROR::".$PROG_NAME.":: Parameter -outputFileList having output file is required!");
	
	print "-fileTestSet = $fileTestSet\n";
	print "-virtualPath = $virtualPath\n";
	print "-outputFileList = $outputFileList\n";
	print "-verbose = $verbose\n";	
	
	print "ok\n";
}	

################################### sub main #####################################
sub main {
	my @families 		= ();
	my $family		= "";
	my $keyword		= "";
	my $sequenceId		= "";
	my $organism		= "";
	my $startAA_endAA = "";
	my $info = "";
	
	@families = FilesIO->fileToArray2d($fileTestSet, "\t");
	for(my $i=0; $i<scalar(@families); $i++) {
		if (index($families[$i][0],"\n") != 0 and index($families[$i][0],"#") != 0){
			$family= $families[$i][0];
			$sequenceId = $families[$i][1];
			$organism = $families[$i][2];
			$startAA_endAA = $families[$i][3];
			$info .=&runFamily($family, $keyword, $sequenceId, $organism, $startAA_endAA, $virtualPath);			
		}
	}
	FilesIO->saveFile($outputFileList, $info, $verbose);		
}

############################ sub main ##################################
sub runFamily {
	my $factor		= $_[0];
	my $keyword			= $_[1];	
	my $sequenceId		= $_[2];
	my $organism		= $_[3];	
	my $startAA_endAA   = $_[4];		
	my $virtualPath		= $_[5];	

	my $value2 = replaceData($factor, $keyword, $sequenceId, $organism, $startAA_endAA, $virtualPath);
	
	return $value2."\n";
	
}			
############################ sub main ##################################
sub replaceData {
	
	my $factor		= $_[0];
	my $keyword		= $_[1];	
	my $sequenceId	= $_[2];
	my $organism	= $_[3];
	my $startAA_endAA= $_[4];	
	my $value2		= $_[5];
	
	#print "replaceDataFamily in ". $value2."\t";
	
	
	if (index(($value2),("FAMILY_FACTOR")) >= 0){
		$value2 = String->replace($value2,"FAMILY_FACTOR", $factor);						
	}	
	
	if (index(($value2),("KEYWORD")) >= 0){
		$value2 = String->replace($value2,"KEYWORD_FACTOR", $keyword); 							
	}
	
	if (index(($value2),("SEQUENCE_ID")) >= 0){
		$value2 = String->replace($value2,"SEQUENCE_ID", $sequenceId); 							
	}
	
	if (index(($value2),("ORGANISM")) >= 0){
		$value2 = String->replace($value2,"ORGANISM", $organism); 							
	}
	
	if (index(($value2),("STARTAA_ENDAA")) >= 0){
		$value2 = String->replace($value2,"STARTAA_ENDAA", $startAA_endAA); 							
	}
	
	#print "out ". $value2."\n";
	return $value2;
}