#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Sequence::sequence;
use Database::PFAM;

my $pfamFile = "";

my $verbose    	= "false";
my $alnDir = "";
my $alnLineSize = 60;



my $PROG_NAME = "CASH_27::scripts::prepare::saveAlignment.pl";

my %hash_aln = ();
my %hash_seq_specie = (); 

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-pfamFile"){
			$pfamFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-alnDir"){
			$alnDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-alnLineSize"){
			$alnLineSize = $ARGV[$counter+1];
		}							
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	String->isEmpty($pfamFile, $PROG_NAME."::ERROR::Parameter -pfamFile having Pfam aligments is required!");
	FilesIO->checkFile($pfamFile, $PROG_NAME."ERROR:: File $pfamFile NOT found ");	
	String->isEmpty($alnDir, $PROG_NAME."::ERROR::Parameter -alnDir having directory to save individual alignments is required!");
	FilesIO->createDirectory($alnDir, $verbose);

	print "-pfamFile = $pfamFile\n";
	print "-alnDir = $alnDir\n";
	print "-alnLineSize = $alnLineSize\n";
	print "-verbose = $verbose\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $line = "";						my @array_aux = ();
	my $accFamily	= "";				my $numSeq = "";
	my $sequence = "";					my $info_aln = "";
	my $sequenceID = "";				my $count_seq = 0;
	my $count_family = 0;				my $accProtNew;
	my $accProtOld;						my $key;
	my %hash_acc = ();					my %hash_specie = ();
	my $count_species = 0;				my $info_list = "";
	my $IDFamily;
	
	
	if (open(FILEIN, "$pfamFile")){
		while (my $line = <FILEIN>) {
			$line =~ s/\n//g;			
			if (index($line, "#=GF AC") == 0){				
				@array_aux = split(/ +/, $line);
				$accFamily = $array_aux[2];	
				$accFamily = PFAM->getAccNumber($accFamily);
			}elsif(index($line, "#=GF SQ") == 0){
				@array_aux = split(/ +/, $line);
				$numSeq = $array_aux[2];
			}elsif(index($line, "#=GF ID") == 0){
				@array_aux = split(/ +/, $line);
				$IDFamily = $array_aux[2];
			}elsif(index($line, "#=GS") == 0){
				@array_aux = split(/ +/, $line);
				if ($array_aux[2] eq "AC"){
					$accProtNew = (split(/\./, $array_aux[3]))[0];
					$accProtOld = (split(/_/, $array_aux[1]))[0];
					$key = $array_aux[1];
					$key =~ s/$accProtOld/$accProtNew/g;
					$hash_acc{$array_aux[1]} = $key;
					
				}
			}elsif(index($line, "//") == 0){
				$count_family++;				
				if ($numSeq != $count_seq){
					print "ERROR::".$PROG_NAME.":: Number of sequence=$count_seq Expected=$numSeq!\n";
					exit;
				}
				$info_list .= "$accFamily\t$numSeq\t$count_species\t$IDFamily\n";
				FilesIO->saveFile($alnDir.$accFamily.".fasta", $info_aln, $verbose);
				$count_seq = 0;
				$info_aln = "";
				%hash_acc = ();
				%hash_specie = ();
				$count_species = 0;
	
				#if ($count_family == 2){
				#	exit;
				#}
			}elsif(index($line, "#") < 0){
				@array_aux = split(/ +/, $line);
				$sequenceID = $array_aux[0];
				if (!exists($hash_acc{$sequenceID})){
					print "ERROR::".$PROG_NAME.":: $sequenceID NOT Found before!\n";
					exit;
				}
				$sequenceID = $hash_acc{$sequenceID};
				$sequenceID =~ s/\//_/g; 
				$sequence = $array_aux[1];
				$sequence =~ s/\./-/g; 
				@array_aux = split(/_/, $sequenceID);
				#count species
				if (!exists($hash_specie{$array_aux[1]})){
					$hash_specie{$array_aux[1]} = "";
					$count_species++;
				}
				
				$sequence = Sequences->divideSeq($sequence, $alnLineSize);
				$info_aln.= ">".$sequenceID."\n".$sequence."\n";
				#print  ">".$sequenceID."\n".$sequence."\n";	
				$count_seq++;			
			}			
		}
		close FILEIN;
		FilesIO->saveFile($pfamFile.".list", $info_list, $verbose);
	}else{
		print "ERROR::".$PROG_NAME.":: File $pfamFile NOT found!\n";
	}	
}


	