#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Sequence::sequence;
use Database::PFAM;
use Util::Hash;

my $PfamNewListFile = "";
my $PfamOldListFile = "";
my $verbose    	= "false";
my $outputFile = "";
my $taxonDir = "";
my $MAXSpecies = 350;
my $clade	= 33630; #Alveolata
my $level = 4;

my $removed = "";
my $QTD = 0;
my $COL_SPECIES = 2;
my $COL_PROT = 1;
my $COL_TAXON = 4;
my $COL_INTERVAL = 3;
my $EXT = ".taxon";

my $TOTAL_SPECIES = 0;

my %hashProtOld = ();
my %hashProtNew = ();
my %hashSpeciesNew = ();
my %hashSelected = ();
my %hashCompTaxon = ();

my $PROG_NAME = "CASH_27::scripts::prepare::selectSpeciesPfam.pl";


&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-PfamNewListFile"){
			$PfamNewListFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-PfamOldListFile"){
			$PfamOldListFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-MAXSpecies"){
			$MAXSpecies = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-taxonDir"){
			$taxonDir = $ARGV[$counter+1];
		}							
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	String->isEmpty($PfamNewListFile, $PROG_NAME."::ERROR::Parameter -PfamNewListFile having Pfam new list is required!");
	FilesIO->checkFile($PfamNewListFile, $PROG_NAME."ERROR:: File $PfamNewListFile NOT found ");		
	String->isEmpty($outputFile, $PROG_NAME."::ERROR::Parameter -outputFile is required!");
	String->isEmpty($taxonDir, $PROG_NAME."::ERROR::Parameter -taxonDir is required!");

	print "-PfamNewListFile = $PfamNewListFile\n";
	print "-PfamOldListFile = $PfamOldListFile\n";
	print "-outputFile = $outputFile\n";
	print "-MAXSpecies = $MAXSpecies\n";
	print "-taxonDir = $taxonDir\n";
	print "-verbose = $verbose\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $partialList = "";			my $partialListNew = "";
	my $specificSpecies = "";			my $newSpecies = "";
	my @keys;

	%hashSpeciesNew = FilesIO->fileToHash($PfamNewListFile, "\t", $COL_SPECIES);
	%hashProtNew = FilesIO->fileToHash($PfamNewListFile, "\t", $COL_PROT);
	&loadTaxonPath;
	
	if (FilesIO->isFile($PfamOldListFile) eq "true"){
		%hashProtOld = FilesIO->fileToHash($PfamOldListFile, "\t", $COL_PROT);
		$partialList = getPartialList();
		%hashSelected = String->stringToHash($partialList, "\n", "\t", $COL_SPECIES);
		#print $partialList;
	}
	print ">>>QTD = $QTD\n";
	if ($QTD < $MAXSpecies){
		if ($removed ne ""){
			$partialListNew = getRemovedSpecies();
		}
		print ">>>QTD = $QTD\n";
		#get Specific species
		%hashSelected = String->stringToHash($partialList.$partialListNew, "\n", "\t", $COL_SPECIES);
		@keys = keys %hashSelected; print "hashSelected = ".scalar(@keys)."\n";
		$specificSpecies = getSpecificSpecies();
		@keys = keys %hashSelected; print "hashSelected = ".scalar(@keys)."\n";
		print "\n====>Alveolata species\n$specificSpecies\n";
		print ">>>QTD = $QTD\n";
		if ($QTD < $MAXSpecies && $QTD < $TOTAL_SPECIES){
			if ($TOTAL_SPECIES < $MAXSpecies){
				$MAXSpecies = $TOTAL_SPECIES;
			}
			$newSpecies = getNewSpecies();
		}	
		@keys = keys %hashSelected; print "hashSelected = ".scalar(@keys)."\n";
	}
	Hash->hashToFileOnlyValue($outputFile, \%hashSelected);

	FilesIO->saveFile($outputFile.".news", $partialListNew.$specificSpecies.$newSpecies, $verbose);
}

############################ sub getNewSpecies ##########################
sub getNewSpecies{
	my @arrayClade = ();		my $count = 0;
	my $speciesTaxonKey;		my $speciesTaxonSelected;
	my $countSpecies = 0;		my $species_value;
	my $info = "";
	my @array_new;				my $info_new;
	my %hashSelectedNew = ();	my @arrayCladeAux = ();
	
	@arrayClade = (); $countSpecies = 0;
	while ( my ($key, $value) = each(%hashSpeciesNew) ) {
			#print "getNewSpecies::$key\n";			
		if (!exists($hashSelected{$key})){
			$speciesTaxonKey = $hashCompTaxon{$key};
			$arrayClade[$countSpecies][0] = $key;
			$count = 0;
			while ( my ($keyS, $valueS) = each(%hashSelected)) {
				$speciesTaxonSelected = $hashCompTaxon{$keyS};				
				if (taxonsCladeIsEqual($speciesTaxonKey, $speciesTaxonSelected, $level) eq "true"){
					$count++;
				}				
			}
			$arrayClade[$countSpecies][1] = $count;
			$countSpecies++;
		}
	}
	
	while ($QTD < $MAXSpecies){
		@arrayClade = Array->sortArray2d(1, \@arrayClade);
		#print Array->printMatrix2("\t", "\n", \@arrayClade);
		
		$species_value = $hashSpeciesNew{$arrayClade[0][0]};
		if (!exists($hashSelectedNew{$arrayClade[0][0]})){
			$hashSelectedNew{$arrayClade[0][0]} = "";
			$info_new  = selectBestSequence($species_value);
			$hashSelected{$arrayClade[0][0]} = $info_new;
			$info .= $info_new;
			print "\n>>>>>Included\t".$arrayClade[0][0]."\t".$info_new;
		}
		@arrayCladeAux = (); $countSpecies = 0;
		for (my $i=1; $i<scalar(@arrayClade); $i++){
			$speciesTaxonKey = $hashCompTaxon{$arrayClade[$i][0]};
			$arrayCladeAux[$countSpecies][0] = $arrayClade[$i][0];
			$arrayCladeAux[$countSpecies][1] = $arrayClade[$i][1];
			
			while ( my ($keyS, $valueS) = each(%hashSelectedNew)) {
				$speciesTaxonSelected = $hashCompTaxon{$keyS};	
				if (taxonsCladeIsEqual($speciesTaxonKey, $speciesTaxonSelected, $level) eq "true"){
					$arrayCladeAux[$countSpecies][1] += 1;
					#print $arrayClade[$i][0]."\t".$speciesTaxonKey."\n $keyS\t$speciesTaxonSelected\n";
				}				
			}
			$countSpecies++;
		}
		#print "CLADE AUX\n". Array->printMatrix2("\t", "\n", \@arrayCladeAux);
		@arrayClade = @arrayCladeAux;		
		$QTD++;	
		print "QTD = $QTD\n";
	}	
	return $info;
}

############################ sub compareTaxons ##########################
sub selectBestSequence{
	my $info		= $_[0];
	
	my @array_new = ();				my $sizeDomain;
	my @array_aux = ();				my $MAX_SIZE = 0;
	my $index = 0;
	
	@array_new = String->StringToArray2d($info, "\n", "\t");	
	if (scalar(@array_new) == 1){
		return $info;
	}else{
		for (my $i=1; $i<scalar(@array_new); $i++){
			@array_aux = split(/-/, $array_new[$i][$COL_INTERVAL]);
			$sizeDomain = ($array_aux[1] - $array_aux[0]) + 1;
			if ($sizeDomain > $MAX_SIZE){
				$MAX_SIZE = $sizeDomain;
				$index = $i;
			}
		}
		return (Array->printMatrixRow("\t", $index, \@array_new)."\n");
	}	
}

############################ sub compareTaxons ##########################
sub taxonsCladeIsEqual{
	my $taxon1		= $_[0];
	my $taxon2		= $_[1];
	my $level		= $_[2];
	
	my @arrayTaxon1 = String->StringToArray($taxon1, "\t");
	my @arrayTaxon2 = String->StringToArray($taxon2, "\t");
	
	for (my $i=0; $i < $level; $i++){
		if ($arrayTaxon1[$i] ne $arrayTaxon2[$i]){
			return "false";
		}
	}
	return "true";
	
}
############################ sub getPartialList ##########################
sub loadTaxonPath{
	my @array_new;			my $taxonID;
	my $taxonFile;			my $taxonInfo;
	
	while ( my ($key, $value) = each(%hashSpeciesNew) ) {
		@array_new = String->StringToArray2d($value, "\n", "\t");
		$taxonID = $array_new[0][$COL_TAXON];
		$taxonFile = $taxonDir.$taxonID.$EXT;
		if (FilesIO->isFile($taxonFile) eq "true"){
			$taxonInfo = FilesIO->lineFileToString($taxonFile);
			$hashCompTaxon{$key} = $taxonInfo;
			$TOTAL_SPECIES++;
		}else{
			print $PROG_NAME."::ERROR:: File $taxonFile NOT found\n";
		}
	}
		
}
############################ sub getPartialList ##########################
sub getPartialList{
	my $info = "";			my @array_new = ();
	my @array_old = ();		my $index = 0;
	
	while ( my ($key, $value) = each(%hashProtOld) ) {
		if (exists($hashProtNew{$key})){
			@array_new = String->StringToArray2d($hashProtNew{$key}, "\n", "\t");
			if (scalar(@array_new) > 1){
				@array_old = String->StringToArray2d($value, "\n", "\t");
				for (my $i=0; $i<scalar(@array_new); $i++){
					if ($array_old[0][$COL_INTERVAL] eq $array_new[0][$COL_INTERVAL]){
						$index = $i;
						$i = scalar(@array_new);
					}
				}
				$info .= Array->printMatrixRow("\t", $index, \@array_new)."\n";
			}else{
				$info .= $hashProtNew{$key};
			}
			
			$QTD++;
		}else{
			print $PROG_NAME."LOG::Sequence $key was removed from the new version\n";
			$removed .=$value;
		}
	}
	return $info;
}

############################ sub getPartialList ##########################
sub getRemovedSpecies{
	my @array_new;		my $info = "";		my $species;
	my $info_new;
	
	my @array_old = String->StringToArray2d($removed, "\n", "\t");
	
	for (my $i = 0; $i < scalar(@array_old); $i++){
		$species = $array_old[$i][$COL_SPECIES];
		
		if (exists($hashSpeciesNew{$species})){
			$info_new  = selectBestSequence($hashSpeciesNew{$species});
			$info .= $info_new;
			$QTD++;
		}
	}
	return $info;
}
############################ sub getPartialList ##########################
sub getSpecificSpecies{
	my @array_new;		my $taxonID;
	my $info = "";		my $taxonFile;
	my $info_new;		my $speciesTaxon;
	
	while ( my ($key, $value) = each(%hashSpeciesNew) ) {
		
		if (!exists($hashSelected{$key})){
			
			$speciesTaxon = $hashCompTaxon{$key};
			if (index($speciesTaxon, $clade) >=0){
					$info_new  = selectBestSequence($value);
					$hashSelected{$key} = $info_new;	
					$info .= $info_new;	
					$QTD++;
			}
		}			
	}
	
	if ($info eq ""){
		print $PROG_NAME."::LOG::WARNING:: No species from $clade was added\n";
	}	
	return $info;
}
############################ sub getPartialList ##########################
sub getPartialListSpecies{
	my $info = "";			my @array_new = ();
	my @array_old = ();		my $index;
	
	while ( my ($key, $value) = each(%hashProtOld) ) {
		@array_old = String->StringToArray2d($value, "\n", "\t");
		if (scalar(@array_old) > 1){
			print $PROG_NAME."::ERROR::OLD PFAM has more than one species: $key\n";
			exit;
		}
		if (exists($hashProtNew{$key})){
			@array_new = String->StringToArray2d($hashProtNew{$key}, "\n", "\t");
			$index = Array->isInColMatrix(1, $array_old[0][$COL_PROT], \@array_new);
			if ($index >-1){
				$info .= Array->printMatrixRow("\t", $index, \@array_new)."\n";
			}else{
				print $PROG_NAME."LOG::Sequence ".$array_old[0][$COL_PROT]." was removed from the new version\n";
			}
		}
	}
	return $info;
}



	