#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Sequence::sequence;
use HMMs::HMMER::hmmer;
use Align::Primary::blast;
use PSSM::psi_blast_source_code;


my $resultFile  = "";
my $verbose    	= "false";
my $tmpDir = "";
my $output = "";
my $familyACC = "";
my $totalModels = "";
my $database  = "";
my $modelFile = "";
my $accNumber_position = 1;
my $accNumber_separator_before = "";
my $accNumber_separator_after = " \|";

my $PROG_NAME = "CASH_27::scripts::run::create_hmm_result_file.pl";	

my %hash_cand_domain = ();

#my @array_ret = HMMER->getAllDomainHmmer3("/Users/julianab/projetos/PFAM_annotation/files/run/log/result.txt", "psu|MAL13P1.6", $verbose);	

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-resultFile"){
			$resultFile = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$output = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-familyACC"){
			$familyACC= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-totalModels"){
			$totalModels= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-database"){
			$database = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-modelFile"){
			$modelFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_position"){
			$accNumber_position= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_separator_before"){
			$accNumber_separator_before= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_separator_after"){
			$accNumber_separator_after= $ARGV[$counter+1];
		}						
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	
	String->isEmpty($resultFile, "ERROR::".$PROG_NAME.":: Parameter -resultFile having HMMER Result file name is required");
	FilesIO->checkFile($resultFile, "ERROR::".$PROG_NAME.":: FILE NOT found ".$resultFile);	
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($output, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required!");
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having Temporary Directory is required!");
	FilesIO->checkDirectory($tmpDir, "ERROR::".$PROG_NAME.":: Directory NOT found ");

			
	print "-resultFile = $resultFile\n";
	print "-outputFile = $output\n";
	print "-verbose = $verbose\n";
	print "-tmpDir = $tmpDir\n";
	print "-accNumber_position = $accNumber_position\n";
	print "-accNumber_separator_before = $accNumber_separator_before\n";
	print "-accNumber_separator_after = $accNumber_separator_after\n";
	print "ok\n";	
	
}	


############################ sub main ##################################
sub main {
	
	my $tf_candidate_name;
	my @array_hits = (); 
	my @array_aux;
	my @array_index;
	my $key_acc;
	
	my $score; 							my $evalue; 
	my $hit_start; 						my $hit_end; 
	my $hitPfamDomain_start; 			my $hitPfamDomain_end;
	my $info = "";						my $tf_candidate_sequence;
	my @arraySeq; 						my $lenQuery;
	my $start_aa;						my $end_aa;							
	my $identity;						my $positivity;
	my @array_blast;					my $alnFile = $tmpDir."aln.txt";
	my @array_ret;
	
	my @array = HMMER->getListIdsHmmer3($resultFile);

	for (my $i = 0; $i < scalar(@array); $i++){
		#$tf_candidate_name = substr($array[$i][0], index($array[$i][0], "psu") + 4, length($array[$i][0]));
		
		$tf_candidate_name = getCandidateName($array[$i]);
		if ($verbose eq "true"){print $tf_candidate_name."\n";}
		$key_acc = getKeyCandidateName($tf_candidate_name, $array[$i]);
		
		if ($verbose eq "true"){print "keyacc $key_acc\n";} 
		@array_ret = HMMER->getAllDomainHmmer3($resultFile, $array[$i], $verbose);		
		@array_ret = Blast->removeOverlapHit(0, 0, "", \@array_ret);
		
		for (my $i = 0; $i < scalar(@array_ret); $i++){
			$evalue 	= $array_ret[$i][0];
			$hit_start	= $array_ret[$i][1];#Hit is match in TF candidate sequence
			$hit_end   	= $array_ret[$i][2];#Hit is match in TF candidate sequence
	
			$hitPfamDomain_start = $array_ret[$i][3];
			$hitPfamDomain_end   = $array_ret[$i][4];
			
			$start_aa = $array_ret[$i][6];
			$end_aa   = $array_ret[$i][7];
		
			FilesIO->saveFile($alnFile, "hit   ".$array_ret[$i][8]. "\nquery    ".$array_ret[$i][9], $verbose);
			$identity = Alignment->getIdentity($alnFile, "clustalw");
			$identity = $identity * 100;
		
			$score = $array_ret[$i][5];
			#print "HMMer-3"."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\n";
			$info .= "HMMer-3"."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\tALL\n";
		}	
	}
	FilesIO->saveFile($output, $info, $verbose);
}

############################ sub main ##################################
sub main2 {
	
	my $tf_candidate_name;
	my $info = "";
	my @array_hits = (); 
	my $evalue;
	my @array_aux;
	my @array_index;
	my $key_acc;
	
	
	$info = HMMER->parserHmmer3Results($resultFile, $verbose);
	my @array =String->StringToArray2d($info, "\n", "\t");
	$info = "";
	for (my $i = 0; $i < scalar(@array); $i++){
		#$tf_candidate_name = substr($array[$i][0], index($array[$i][0], "psu") + 4, length($array[$i][0]));
		
		$tf_candidate_name = getCandidateName($array[$i][0]);
				#print $array_hits[$j][1]."\t".$array_hits[$j][2]."\n";
		if ($verbose eq "true"){print $tf_candidate_name."\n";}
		$key_acc = getKeyCandidateName($tf_candidate_name, $array[$i][0]);
		
		if ($verbose eq "true"){print "keyacc $key_acc\n";} 
		if (!exists($hash_cand_domain{$tf_candidate_name})){
			$hash_cand_domain{$tf_candidate_name} = "";
			$info .= runHMMERHitByQuery($tf_candidate_name, $key_acc);	
		}	
	}
	FilesIO->saveFile($output, $info, $verbose);
}

############################ sub main ##################################
sub getCandidateName {
	my $name 	= $_[0];	
	
	my $tf_candidate_name = $name;
	
	if (index($name, "\|") >= 0){
		my @array_aux = split(/\|/, $name);
		my @array_index = split(/;/, $accNumber_position);
		$tf_candidate_name = "";
		for (my $k = 0; $k < scalar(@array_index); $k++){
			$tf_candidate_name .= $array_aux[$array_index[$k]];
			if ($verbose eq "true"){print $tf_candidate_name."\n";}
			if ($k != scalar(@array_index)-1){
				$tf_candidate_name .= "|";
			}
		}
	}	
	return $tf_candidate_name;	
}

############################ sub main ##################################
sub getKeyCandidateName {
	my $name 	= $_[0];	
	my $name_old 	= $_[1];
	my $key;
	if (index($name_old, "\|") >= 0){
		$key = $accNumber_separator_before.$name.$accNumber_separator_after;
	}else{
		$key = $name;
	}
	if ($verbose eq "true"){print "key $key\n";} 
	return $key;
}

############################ sub runBlastHitBySpecie ####################################
sub runHMMERHitByQuery{

	my ($tf_candidate_name, $key_acc) = @_;
	
	my $tf_candidate_name_file = String->removeCaracter($tf_candidate_name, "\|");
	my $result_file_tf_candidate = $tmpDir.$tf_candidate_name_file.".result";
	
	my $score; 							my $evalue; 
	my $hit_start; 						my $hit_end; 
	my $hitPfamDomain_start; 			my $hitPfamDomain_end;
	my $info = "";						my $tf_candidate_sequence;
	my @arraySeq; 						my $lenQuery;
	my $start_aa;						my $end_aa;							
	my $identity;						my $positivity;
	my @array_blast;					my $alnFile = $tmpDir."aln.txt";
	
	
	$tf_candidate_sequence = $tmpDir."seq.out.$tf_candidate_name_file"; 
	Sequences->getOneSequenceFromDatabase($database, $tf_candidate_sequence, $key_acc, $verbose);
	@arraySeq = Sequences->getSequenceFromFastaArray2D($tf_candidate_sequence);
	$lenQuery =  Sequences->lengthSequence($arraySeq[0][1]);	
	
	HMMER->search($modelFile, $result_file_tf_candidate, $tf_candidate_sequence," -E 10 --max ", "", $verbose);
	my @array_ret = HMMER->getAllDomainHmmer3($result_file_tf_candidate, $tf_candidate_name, $verbose);		
	@array_ret = Blast->removeOverlapHit(\@array_ret);
	
	for (my $i = 0; $i < scalar(@array_ret); $i++){
		$evalue 	= $array_ret[$i][0];
		$hit_start	= $array_ret[$i][1];#Hit is match in TF candidate sequence
		$hit_end   	= $array_ret[$i][2];#Hit is match in TF candidate sequence
	
		$hitPfamDomain_start = $array_ret[$i][3];
		$hitPfamDomain_end   = $array_ret[$i][4];

		$start_aa = $array_ret[$i][6];
		$end_aa   = $array_ret[$i][7];
		
		if (index($array_ret[$i][9], " ") >= 0){
			print "ERROR:: $modelFile, $result_file_tf_candidate \n";
		}
		
		FilesIO->saveFile($alnFile, "hit   ".$array_ret[$i][8]. "\nquery    ".$array_ret[$i][9]);
		$identity = Alignment->getIdentity($alnFile, "clustalw");
		$identity = $identity * 100;
		
		$score = $array_ret[$i][5];
	
		#print "HMMer-3"."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\n";
		$info .= "HMMer-3"."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\tALL\n";
	}	
		
	return $info;		
}
