#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Sequence::sequence;
use HMMs::HMMER::hmmer;

my $pfamFile = "";

my $verbose    	= "false";
my $modelDir = "";
my $trainFile = "";
my $tmpDir = "";
my $iteration = 5;
my $database = "";
my $parameters = "";
my $evalue = 0.001;


my $PROG_NAME = "CASH_v26::scripts::run::create_models.pl";

my %hash_aln = ();
my %hash_seq_specie = (); 

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-pfamFile"){
			$pfamFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-modelDir"){
			$modelDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-trainFile"){
			$trainFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-iteration"){
			$iteration = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-evalue"){
			$evalue = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-parameters"){
			$parameters = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-database"){
			$database = $ARGV[$counter+1];
		}								
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	String->isEmpty($pfamFile, $PROG_NAME."::ERROR::Parameter -pfamFile having Pfam aligments is required!");
	FilesIO->checkFile($pfamFile, $PROG_NAME."ERROR:: File $pfamFile NOT found ");	
	String->isEmpty($modelDir, $PROG_NAME."::ERROR::Parameter -modelDir having directory to save probabilistic models is required!");
	FilesIO->createDirectory($modelDir, $verbose);
	String->isEmpty($trainFile, $PROG_NAME."::ERROR::Parameter -trainFile having training data file is required!");
	FilesIO->checkFile($trainFile, $PROG_NAME."ERROR:: File $trainFile NOT found ");	
	String->isEmpty($database, $PROG_NAME."::ERROR::Parameter -database having database in fasta format is required!");
	FilesIO->checkFile($database, $PROG_NAME."ERROR:: File $database NOT found ");	
	String->isEmpty($tmpDir, $PROG_NAME."::ERROR::Parameter -tmpDir having temporary directory is required!");
	FilesIO->createDirectory($tmpDir, $verbose);
	
	print "-pfamFile = $pfamFile\n";
	print "-modelDir = $modelDir\n";
	print "-trainFile = $trainFile\n";
	print "-database = $database\n";
	print "-evalue = $evalue\n";
	print "-iteration = $iteration\n";
	print "-tmpDir = $tmpDir\n";
	print "-verbose = $verbose\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $acc = "";						my @array_aux = ();
	my $specie	= "";					my $interval = "";
	my $key = "";						my $query_sequence;
	my $modelName;						my $query_file;
	my $count = 0;						my $progress;
	my $modelNameTmp;
	
	my %hash_acc = FilesIO->fileToHashConcatKey($trainFile, "\t", "1,2,3");
	my @sequences = Sequences->getSequenceFromFastaArray2D($pfamFile);
	my $total = keys( %hash_acc );
	
	for (my $i=0; $i < scalar(@sequences); $i++){
	 	@array_aux = split(/_/, $sequences[$i][0]);
	 	$acc = String->removeCaracter($array_aux[0], ">");
	 	$specie = $array_aux[1];
	 	$interval = String->removeCaracter($array_aux[2], "\n");
	 	$key = $acc.$specie.$interval;
	 	#print " $key\n";
	 	if (exists($hash_acc{$key})){
	 		$count++;
			#create model
			$modelName = $modelDir.$acc."_".$specie."_".$interval.".hmm";
			$modelNameTmp = $tmpDir.$acc."_".$specie."_".$interval.".hmm";
			if (FilesIO->isFile($modelName) eq "true"){
				print $PROG_NAME."::LOG::File $modelName already created.\n";
			}
			else{
				$query_sequence = $sequences[$i][0].Sequences->divideSeq(String->removeCaracter($sequences[$i][1], "-"), 60);
				$query_file = $tmpDir.$acc."_".$specie."_".$interval.".fasta";
				FilesIO->saveFile($query_file, $query_sequence, $verbose);
				print $query_sequence."\n";
				HMMER->runJackHmmer($modelNameTmp, $query_file, $database, $parameters, $iteration, $evalue, $verbose);
				FilesIO->copyFile($modelNameTmp, $modelName, $verbose);
			}
			print "create model $acc $specie $interval\n";
			$progress = $count/$total;
			
			print "LOG::PROG:: $pfamFile == ".sprintf("%.2f", $progress)."\n";
		}
		
	}
}


	