#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Sequence::sequence;
use Align::Primary::blast;
use PSSM::psi_blast;
use Util::HTML;
use Util::Array;
use PSSM::psi_blast_source_code;

my $fileList  = "";
my $verbose    	= "false";
my $database  = "";
my $tmpDir = "";
my $output = "";
my $familyACC = "";
my $totalModels = "";
my $accNumber_position = 1;
my $accNumber_separator_before = "";
my $accNumber_separator_after = " \|";

my $MAX_EVALUE = "1e-05";

my $PROG_NAME = "CASH_27::scripts::run::create_pssm_result_file.pl";	

my %hash_cand_domain = ();


&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-fileList"){
			$fileList = $ARGV[$counter+1];		
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-database"){
			$database = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$output = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-familyACC"){
			$familyACC= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_position"){
			$accNumber_position= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_separator_before"){
			$accNumber_separator_before= $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-accNumber_separator_after"){
			$accNumber_separator_after= $ARGV[$counter+1];
		}				
	}
}

############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters for $PROG_NAME\n";
	

	String->isEmpty($fileList, "ERROR::".$PROG_NAME.":: Parameter -fileList having files to compare is required!");
	FilesIO->checkFile($fileList, "ERROR::".$PROG_NAME.":: FILE NOT found".$fileList);	
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($output, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required!");
	String->isEmpty($database, "ERROR::".$PROG_NAME.":: Parameter -database having database file is required!");
	FilesIO->checkFile($database, "ERROR::".$PROG_NAME.":: FILE NOT found ".$database);	
	String->isEmpty($tmpDir, "ERROR::".$PROG_NAME.":: Parameter -tmpDir having Temporary Directory is required!");
	FilesIO->checkDirectory($tmpDir, "ERROR::".$PROG_NAME.":: Directory NOT found ");
	
	String->isEmpty($familyACC, "ERROR::".$PROG_NAME.":: Parameter -familyACC having PFAM Accession number is required!");

	
	print "-fileList = $fileList\n";
	print "-outputFile = $output\n";
	print "-verbose = $verbose\n";
	print "-database = $database\n";
	print "-tmpDir = $tmpDir\n";
	print "-familyACC = $familyACC\n";
	#print "-totalModels = $totalModels\n";
	print "-accNumber_position = $accNumber_position\n";
	print "-accNumber_separator_before = $accNumber_separator_before\n";
	print "-accNumber_separator_after = $accNumber_separator_after\n";
	print "ok\n";	
}	
############################ sub main ##################################
sub getCandidateName {
	my $name 	= $_[0];	
	
	my $tf_candidate_name = $name;
	
	if (index($name, "\|") >= 0){
		my @array_aux = split(/\|/, $name);
		my @array_index = split(/;/, $accNumber_position);
		$tf_candidate_name = "";
		for (my $k = 0; $k < scalar(@array_index); $k++){
			$tf_candidate_name .= $array_aux[$array_index[$k]];
			if ($verbose eq "true"){print $tf_candidate_name."\n";}
			if ($k != scalar(@array_index)-1){
				$tf_candidate_name .= "|";
			}
		}
	}	
	return $tf_candidate_name;	
}




############################ sub main ##################################
sub main {
	my @array =  FilesIO->fileToArray2d($fileList, ":");
	
	my $result_file;					my $tf_candidate_name;
	my $pfam_sequence_name;				my $pssmFile;
	my $info = "";						my @array_hits = (); 
	my @array_aux;						my $start_end_aa;
	my $specie;							my $start_aa;
	my $end_aa;							my $evalue;
	my $hit_start;						my $hit_end;
	my $score;							my $identity;
	
	for (my $i = 0; $i < scalar(@array); $i++){
		if ($verbose eq "true"){print "LOG::$i\n"};
		$result_file = $array[$i][1];
		$pfam_sequence_name = $array[$i][3];
		$pssmFile =  $array[$i][0];
		$start_end_aa = $array[$i][4];
		@array_aux = split(/-/,$start_end_aa);
		$start_aa = $array_aux[0];
		$end_aa   = $array_aux[1];
		$specie = $array[$i][5];
		%hash_cand_domain = ();
		if (FilesIO->isFile($result_file) eq "true"){
			@array_hits = FilesIO->fileToArray2d($result_file, "\t");
			if ($verbose eq "true"){print $result_file."\n";}
			#print scalar(@array_hits)."\n";
			for (my $j = 0; $j < scalar(@array_hits); $j++){
				#$tf_candidate_name = substr($array_hits[$j][1], index($array_hits[$j][1], "psu") + 4, length($array_hits[$j][1])); 	
				if ($verbose eq "true"){print $array_hits[$j][1]."\n";}
				$tf_candidate_name = getCandidateName($array_hits[$j][1]);	
				$evalue = $array_hits[$j][10];
				$hit_start = $array_hits[$j][8];
				$hit_end = $array_hits[$j][9];
				$score =  $array_hits[$j][11];
				$identity = $array_hits[$j][2];
				$info .= $pfam_sequence_name."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\t".$specie."\n";						
			}
		}elsif ($verbose eq "true"){
				"WARMINIG::".$PROG_NAME.":: File $result_file NOT found\n";			
		}		
	}
	FilesIO->saveFile($output, $info, $verbose);
}
############################ sub main ##################################
sub main2 {
	my @array =  FilesIO->fileToArray2d($fileList, ":");
	
	my $result_file;
	my $tf_candidate_name; my $pfam_sequence_name;
	my $pssmFile;
	my $info = "";
	my @array_hits = (); 
	my @array_aux;
	my @array_index;
	my $start_end_aa;
	my $key_acc;
	my $specie;
	
	for (my $i = 0; $i < scalar(@array); $i++){
		if ($verbose eq "true"){print "LOG::$i\n"};
		$result_file = $array[$i][1];
		$pfam_sequence_name = $array[$i][3];
		$pssmFile =  $array[$i][0];
		$start_end_aa = $array[$i][4];
		$specie = $array[$i][5];
		%hash_cand_domain = ();
		if (FilesIO->isFile($result_file) eq "true"){
			@array_hits = FilesIO->fileToArray2d($result_file, "\t");
			if ($verbose eq "true"){print $result_file."\n";}
			#print scalar(@array_hits)."\n";
			for (my $j = 0; $j < scalar(@array_hits); $j++){
				#$tf_candidate_name = substr($array_hits[$j][1], index($array_hits[$j][1], "psu") + 4, length($array_hits[$j][1])); 	
				if ($verbose eq "true"){print $array_hits[$j][1]."\n";}
				$tf_candidate_name = getCandidateName($array_hits[$j][1]);
				#print $array_hits[$j][1]."\t".$array_hits[$j][2]."\n";
				if ($verbose eq "true"){print $tf_candidate_name."\n";}
				$key_acc = getKeyCandidateName($tf_candidate_name, $array_hits[$j][1]);
				if ($verbose eq "true"){print "keyacc $key_acc\n";} 
				if (!exists($hash_cand_domain{$tf_candidate_name})){
					$hash_cand_domain{$tf_candidate_name} = "";
					$info .= runPsiBlastHitByQuery($pfam_sequence_name, $pssmFile, $tf_candidate_name, $key_acc, $totalModels, $start_end_aa, $specie);
				}						
			}
		}elsif ($verbose eq "true"){
				"WARMINIG::".$PROG_NAME.":: File $result_file NOT found\n";			
		}		
	}
	FilesIO->saveFile($output, $info, $verbose);
}

############################ sub main ##################################
sub getKeyCandidateName {
	my $name 	= $_[0];	
	my $name_old 	= $_[1];
	my $key;
	if (index($name_old, "\|") >= 0){
		$key = $accNumber_separator_before.$name.$accNumber_separator_after;
	}else{
		$key = $name;
	}
	if ($verbose eq "true"){print "key $key\n";} 
	return $key;
}

############################ sub runBlastHitBySpecie ####################################
sub runPsiBlastHitByQuery {
	my ($pfam_sequence_name, $pssmFile, $tf_candidate_name, $key_acc, $totalQueries, $start_end_aa, $specie) = @_;
	my $info = "";
	my $tf_candidate_name_file = String->removeCaracter($tf_candidate_name, "\|");
	my $output_blast = $tmpDir."blast.out.$pfam_sequence_name.$tf_candidate_name_file";
	my @array_ret = ();
	
	my $tf_candidate_sequence = $tmpDir."seq.out.$tf_candidate_name_file";  
	

	Sequences->getOneSequenceFromDatabase($database, $tf_candidate_sequence, $key_acc, $verbose);
	PSIBLAST2->formatdb($tf_candidate_sequence, $verbose);
	PSIBLAST2->searchPSSM($tf_candidate_sequence, $pssmFile, $output_blast, " -num_iterations 1 -evalue 10 -outfmt \"6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore ppos \" ", $verbose);
	
	my @array_aux = ();					my $evalue;
	my $hit;							my $hit_start;
	my $hit_end;						my $hitPfamDomain;
	my $hitPfamDomain_end;				my $start_aa;
	my $end_aa;							my $score;
	my $identity;						my $positivity;
	my $hitPfamDomain_start;
	
	@array_ret  = Blast->getBlastHitFromFileFormated($output_blast, "true");
	for (my $i = 0; $i < scalar(@array_ret); $i++){
		$evalue 	= $array_ret[$i][0];
		$hit_start	= $array_ret[$i][1];#Hit is match in TF candidate sequence
		$hit_end   	= $array_ret[$i][2];#Hit is match in TF candidate sequence
	
		$hitPfamDomain_start = $array_ret[$i][3];
		$hitPfamDomain_end   = $array_ret[$i][4];
	
		@array_aux = split(/-/,$start_end_aa);
		$start_aa = $array_aux[0];
		$end_aa   = $array_aux[1];
	
		$score = $array_ret[$i][5];
		$identity = $array_ret[$i][6];
		$positivity = $array_ret[$i][7];
		$info .= $pfam_sequence_name."\t".$start_aa."\t".$end_aa."\t".$tf_candidate_name."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$score."\t".$identity."\t".$specie."\n";
	}	
	return $info;
}

############################ sub runBlastHitBySpecie ####################################
sub runPsiBlastHitByQueryBackup {
	my ($pfam_sequence_name, $pssmFile, $tf_candidate_name, $totalQueries) = @_;
	my $info = "";
	my $tf_candidate_name_file = String->removeCaracter($tf_candidate_name, "\|");
	my $output_blast = $tmpDir."blast.out.$pfam_sequence_name.$tf_candidate_name_file";
	my @array_ret = ();
	
	my $tf_candidate_sequence = $tmpDir."seq.out.$tf_candidate_name_file";  

	Sequences->getOneSequenceFromDatabase($database, $tf_candidate_sequence, $accNumber_separator_before.$tf_candidate_name.$accNumber_separator_after, $verbose);
	PSIBLAST2->formatdb($tf_candidate_sequence, $verbose);
	PSIBLAST2->searchPSSM($tf_candidate_sequence, $pssmFile, $output_blast, " -num_iterations 1 -outfmt \"6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore ppos \" ", $verbose);
	
	my @arraySeq = Sequences->getSequenceFromFastaArray2D($tf_candidate_sequence);
	my $lenQuery =  Sequences->lengthSequence($arraySeq[0][1]);

	@array_ret  = Blast->getBlastHitFromFileFormated($output_blast);
	my @array_aux = ();
	my $evalue = $array_ret[0][0];
	my $hit = $array_ret[0][1]; #Hit is match in TF candidate sequence
	@array_aux = split(/-/,$hit);
	my $hit_start = $array_aux[0];
	my $hit_end   = $array_aux[1];
	
	my $hitPfamDomain = $array_ret[0][2];
	@array_aux = split(/-/,$hitPfamDomain);
	my $hitPfamDomain_start = $array_aux[0];
	my $hitPfamDomain_end   = $array_aux[1];
	
	my $score = $array_ret[0][3];
	#$info = $familyACC."\t".$pfam_sequence_name."\t".$tf_candidate_name."\t".$lenQuery."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$hitPfamDomain_start."\t".$hitPfamDomain_end."\t".$score."\t".$totalQueries."\n";
	$info = $pfam_sequence_name."\t".$tf_candidate_name."\t".$lenQuery."\t".$evalue."\t".$hit_start."\t". $hit_end."\t".$hitPfamDomain_start."\t".$hitPfamDomain_end."\t".$score."\t".$totalQueries."\n";
	return $info;
}

