#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){ use lib $ENV{'PERL_LIB'};}else{print "ERROR:: Not existis PERL_LIB\n";exit;}

use File::Files;
use Sequence::sequence;
use PSSM::psi_blast_source_code;

my $pfamFile = "";

my $verbose    	= "false";
my $modelDir = "";
my $trainFile = "";
my $tmpDir = "";
my $database = "";
my $PFAM_ACC = "";
my $seqDir = "";


my $PROG_NAME = "CASH_27::scripts::run::save_sequences_running.pl";

my %hash_aln = ();
my %hash_seq_specie = (); 

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-pfamFile"){
			$pfamFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-modelDir"){
			$modelDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-trainFile"){
			$trainFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-tmpDir"){
			$tmpDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-PFAM_ACC"){
			$PFAM_ACC = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-seqDir"){
			$seqDir = $ARGV[$counter+1];
		}									
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEqual($verbose,"true,false",$PROG_NAME."::ERROR::Parameter -verbose must be true or false!");
	String->isEmpty($pfamFile, $PROG_NAME."::ERROR::Parameter -pfamFile having Pfam aligments is required!");
	FilesIO->checkFile($pfamFile, $PROG_NAME."ERROR:: File $pfamFile NOT found ");	
	String->isEmpty($modelDir, $PROG_NAME."::ERROR::Parameter -modelDir having directory to save probabilistic models is required!");
	FilesIO->createDirectory($modelDir, $verbose);
	String->isEmpty($trainFile, $PROG_NAME."::ERROR::Parameter -trainFile having training data file is required!");
	FilesIO->checkFile($trainFile, $PROG_NAME."ERROR:: File $trainFile NOT found ");	
	String->isEmpty($tmpDir, $PROG_NAME."::ERROR::Parameter -tmpDir having temporary directory is required!");
	FilesIO->createDirectory($tmpDir, $verbose);
	String->isEmpty($PFAM_ACC, $PROG_NAME."::ERROR::Parameter -PFAM_ACC having PFAM accession number is required!");
	
	print "-pfamFile = $pfamFile\n";
	print "-modelDir = $modelDir\n";
	print "-trainFile = $trainFile\n";
	print "-database = $database\n";
	print "-tmpDir = $tmpDir\n";
	print "-verbose = $verbose\n";
	print "-PFAM_ACC = $PFAM_ACC\n";
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $acc = "";						my @array_aux = ();
	my $specie	= "";					my $interval = "";
	my $key = "";						my $query_sequence;
	my $modelName;						my $query_file;
	my $name_seq;						my $progress;
	my $modelNameTmp;					my $sequence;
	
	my %hash_acc = FilesIO->fileToHashConcatKey($trainFile, "\t", "1,2,3");
	my @sequences = Sequences->getSequenceFromFastaArray2D($pfamFile);
	my $total = keys( %hash_acc );
	
	for (my $i=0; $i < scalar(@sequences); $i++){
	 	@array_aux = split(/_/, $sequences[$i][0]);
	 	$acc = String->removeCaracter($array_aux[0], ">");
	 	$specie = $array_aux[1];
	 	$interval = String->removeCaracter($array_aux[2], "\n");
	 	$key = $acc.$specie.$interval;
	 	#print " $key\n";
	 	if (exists($hash_acc{$key})){	 		
			#create model
			$modelName = $modelDir.$acc."_".$specie."_".$interval.".chk";
			$name_seq = $acc."_".$specie."_".$interval;
			$modelNameTmp = $tmpDir.$acc."_".$specie."_".$interval.".chk";
			if (FilesIO->isFile($modelName.".tar.gz") eq "true"){
				print $PROG_NAME."::LOG::File $modelName.tar.gz already created.\n";
			}
			else{
				$sequence = String->removeCaracter($sequences[$i][1], "-");
				if (length($sequence) <= 30){
					print $PROG_NAME."::WARNING::Sequence $name_seq is too small .\n";
				}else{
					$query_sequence = $sequences[$i][0].Sequences->divideSeq($sequence, 60);
					$query_file = $seqDir.$acc."_".$specie."_".$interval.".fasta";
					FilesIO->saveFile($query_file, $query_sequence, $verbose);
					print "LOG::LIST::".$PFAM_ACC."\t".$name_seq."\n";
				}				
			}
		}		
	}
}


	