#!/usr/bin/perl -w
use strict;
if (exists($ENV{'PERL_LIB'})){
	use lib $ENV{'PERL_LIB'};
} 
else{
	print "ERROR:: Not existis PERL_LIB\n";
	exit;
}

use File::Files;
use Database::SwissProt;
use Sequence::sequence;
use Sequence::protein;
use Util::Hash;

my $outputFile = "";
my $verbose    	= "false";
my $archFile  = "";
my $sequenceFile = "";
my $listDir  = "";
my $fieldPfam = 1;


my $sep = 1; # 1=space, 2=tab or other 
my $fieldAcc = 0;
my $PROG_NAME = "CASH_27::scripts::ensemble::put_taxonCode.pl";


	

&getParameters;
&checkParameters;
&main;


############################ sub getParameters ##################################
sub getParameters {
	my $counter;
	my $ending_value = scalar(@ARGV);		#numero de parametros de entrada
	for($counter = 0 ; $counter < $ending_value ; $counter++){
		if($ARGV[$counter] eq "-verbose"){
			$verbose = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-outputFile"){
			$outputFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-listDir"){
			$listDir = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-sequenceFile"){
			$sequenceFile = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-sep"){
			$sep = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-fieldAcc"){
			$fieldAcc = $ARGV[$counter+1];
		}elsif($ARGV[$counter] eq "-fieldPfam"){
			$fieldPfam = $ARGV[$counter+1];
		}			
	}
}
############################ sub checkParameters ##################################
sub checkParameters {
	print "\n=============================================\n";
	print "Cheking Parameters $PROG_NAME\n";
	
	String->isEmpty($outputFile, "ERROR::".$PROG_NAME.":: Parameter -outputFile having output file is required");
	String->isEqual($verbose,"true,false", "ERROR::".$PROG_NAME.":: Parameter -verbose must be true or false!");
	String->isEmpty($sequenceFile, "ERROR::".$PROG_NAME."::Parameter -sequenceFile having sequence file in FASTA format is required");
	FilesIO->checkFile($sequenceFile, "ERROR::".$PROG_NAME.":: File NOT found ".$sequenceFile);	
	
	if ($sep eq "1"){
		$sep = " +";
	}elsif ($sep eq "2"){
		$sep = "\t";
	}
	
	print "-archFile = $archFile\n";
	print "-outputFile = $outputFile\n";
	print "-listDir = $listDir\n";
	print "-verbose = $verbose\n";
	print "-sequenceFile = $sequenceFile\n";
	print "-sep = $sep\n";	
	print "-fieldAcc = $fieldAcc\n";	
	
	print "ok\n";	
}	

############################ sub main ##################################
sub main{
	my $accNumber;				my $ann;
	my $seq;					my $interval;
	my @array_range;			my $start;
	my $end;					my $sequence;
	my $count = 1;				my $info = "";
	my $new_ann;				my $taxonCode;
	my @array_aux = ();			my $infoPfam;
	my $pfamCode;				my $listFile;
		
	
	my @array_seq = Sequences->getSequenceFromFastaArray2D($sequenceFile); 
	
	for (my $i=0; $i < scalar(@array_seq); $i++){
		$ann = $array_seq[$i][0];
		$seq = $array_seq[$i][1];
		$ann =  String->removeCaracter($ann, "\n");
		$accNumber = String->getElementArrayLine($ann, $sep, $fieldAcc);
		$accNumber =  String->removeCaracter($accNumber, ">");
		#@array_aux = split(/_/, $accNumber);
		#$accNumber = $array_aux[0];
		$pfamCode = String->getElementArrayLine($ann, $sep, $fieldPfam);
		$listFile = $listDir.$pfamCode.".list";
		#print $listFile."\n";
		if (FilesIO->isFile($listFile) eq "true"){
			$infoPfam = FilesIO->getLineInFileFirstOcurrence($listFile, $accNumber, $verbose);
			if ($infoPfam ne "") {
				@array_aux = String->StringToArray2d($infoPfam, "\n", "\t");
				$taxonCode = $array_aux[0][4];
				$taxonCode =  String->removeCaracter($taxonCode, "\n");
				$sequence = String->removeCaracter($array_seq[$i][1], "\n");
				$new_ann = $ann."\t".$taxonCode."\n";
				$info .= $new_ann.uc(Sequences->divideSeq($sequence, 60))."\n";	
			}
			else{
				print "ERROR::".$PROG_NAME."::$listFile NOT found\n";
			}						
		}else{
			print "ERROR::".$PROG_NAME."::$accNumber NOT found in $archFile\n";
			exit;
		}		
	}
	if ($info ne ""){
		FilesIO->saveFile($outputFile, $info, $verbose);
	}	
}
	